//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// デバッガ (M680x0 依存部分)
//

#pragma once

#include "debugger_private.h"
#include "mpu680x0.h"
#include "m68030.h"
#include "m68030disasm.h"

class DebuggerMD_m680x0 : public DebuggerMD
{
	using inherited = DebuggerMD;
 public:
	DebuggerMD_m680x0(Debugger *parent, m68kcpu *cpu);
	~DebuggerMD_m680x0() override;

	void ReqSet(uint32 flag) override { /*TODO*/ }
	void ReqClr(uint32 flag) override { /*TODO*/ }

	bool IsSuper() const override { return cpu->reg.s; }
	bool MMUEnabled() const override { return cpu->mmu_enable; }
	uint64 TranslateAddr(saddr_t laddr, bool lookup) const override;

	uint32 GetPC() const override { return RegPC; }

	void SetStepOut() override;
	bool IsStepOut() const override;
	bool IsOpStepIn(DebuggerMemoryStream& mem) override;
	uint64 GetRegAddr(const char *name) const override;
	void BackupRegs() override { prev = cpu->reg; }
	bool ShowRegister(FILE *, const std::vector<std::string>& args) override;
	const HelpMessages& GetHelpListReg() const override;
	const HelpMessages& GetHelpReg() const override;

	BranchHistory& GetBrHist() const override { return cpu->brhist; }
	BranchHistory& GetExHist() const override { return cpu->exhist; }

	bool CheckLEA(uint32 laddr) override { return false; }

	bool Disassemble(DebuggerMemoryStream& mem,
		std::string& mnemonic, std::vector<uint8>& bin) override;
	std::string FormatDisasm(const std::string& mnemonic,
		const std::vector<uint8>& bin) override;
	std::string FormatDisasmLive(const std::string& mnemonic,
		const std::vector<uint8>& bin) override;

 private:
	m68kcpu *cpu {};		// CPU コア
	m68kreg prev {};		// 前回のレジスタセットのコピー

	const char *CondStr(const std::vector<uint16>& ir);
	bool IsOPcc(uint16 op);
	bool IsDBcc(uint16 op);
	bool IsCond(uint16 op);

	static const HelpMessages HelpListReg;
	static const HelpMessages HelpReg;
	void ShowRegMain(FILE *);
	void ShowRegFPU(FILE *);
	void ShowRegMMU(FILE *);
	void ShowRegATC();
	void ShowRegOther();

	uint32 so_a7 {};			// ステップアウト判定用
	uint16 so_sr {};
};
