//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// ベクタテーブル (モニタ)
//

#pragma once

#include "object.h"
#include "monitor.h"
#include <array>
#include <vector>

class VectorTable : public Object
{
	using inherited = Object;

 public:
	VectorTable(VMType vmtype);
	~VectorTable() override;

	// 例外名を返す (ベクタテーブル用)
	const char *GetTableName(int vector) const;

	// 例外名を返す (例外履歴用)
	const char *GetExceptionName(int vector) const;

	// ベクタテーブルの要素数を返す
	int Size() const { return nametable.size(); }

	// モニタ更新
	DECLARE_MONITOR_CALLBACK(MonitorUpdate);

 private:
	// 初期化
	void InitTableM68030(VMType vmtype);
	void InitTableLuna88k();

	// ベクタテーブル
	std::vector<const char *> nametable {};

	// モニタ
	Monitor monitor { this };

	// 初期値
	static std::array<const char * const, 64>  name_m68030;
	static std::array<const char * const, 256> name_x68030;
	static std::array<const char * const, 7>   name_luna1;
	static std::array<const char * const, 11>  name_luna88k;
};

extern VectorTable *gVectorTable;
