<!--
 nono
 Copyright (C) 2020 nono project
 Licensed under nono-license.txt
-->
<html><?php
	$nono_ver = "@version@";
	$nono_date = "@date@";

	$fp = fopen("../lib/header.h", "r");
	if ($fp !== false) {
		while (($buf = fgets($fp)) !== false) {
			if (preg_match("/NONO_MAJOR_VER\s+\((\d+)\)/", $buf, $m)) {
				$nono_major = $m[1];
			}
			if (preg_match("/NONO_MINOR_VER\s+\((\d+)\)/", $buf, $m)) {
				$nono_minor = $m[1];
			}
			if (preg_match("/NONO_PATCH_VER\s+\((\d+)\)/", $buf, $m)) {
				$nono_patch = $m[1];
			}
			if (preg_match("/NONO_DATE\s+\"([^\"]+)\"/", $buf, $m)) {
				$nono_date = $m[1];
			}
		}
		fclose($fp);

		$nono_ver = "${nono_major}.${nono_minor}.${nono_patch}";
	}
?>
<?php require_once "common.php"; ?>
<body>
<h3>nono <?=$nono_ver?> (<?=$nono_date?>)</h3>

nono は NetBSD とかで動作する OMRON LUNA-I/LUNA-88K
(と SHARP X68030) のエミュレータです。
<q>nono is OMRON LUNA-I/LUNA-88K (and SHARP X68030) emulator
runs on NetBSD and etc.</q>

<hr>

<h4>Index of this page:</h4>
<div class="main">
<a href="#build">1. ビルド方法 <q>How to build</q></a><br>
<a href="#execute">2. 実行方法 <q>How to execute</q></a><br>
<a href="#configuration">3. 設定 <q>Configuration</q></a><br>
<a href="#aboutvm">4. VM について <q>About VM</q></a><br>
<a href="#tryit">5. 実行してみる <q>Try it</q></a><br>
<a href="#network">6. ホストネットワーク設定例 <q>Example of host network setup</q></a><br>
<a href="#knownissues">7. 既知の問題 <q>Known Issues</q></a><br>
<a href="#migrate">8. 過去のバージョンからの移行方法
<q>How to migrate from old versions</q></a><br>
<a href="#changes">9. 変更履歴 <q>Changes</q></a><br>
<a href="#license">10. 連絡先、ライセンス等 <q>Contact, License, etc</q></a><br>
</div>
<p>
<div class="main">
<span class="new">緑背景</span>は新規または目立った更新のあった箇所です。
<q><span class="new">Green Background</span> is new or updated paragraph.</q>
</div>


<a name="build"></a>
<h4>1. ビルド方法 <q>How to build</q></h4>
<div class="main">
ビルドには以下が必要です。
<q>The followings are required for build.</q>

<ul>
<li>make (BSD make, not GNU make)
<li>C/C++ compiler which supports -std=c++14.
<br>
(For gcc, 7.4 or newer works at least.
For clang, 7.0 - 13.0 works at least.)
<li>wxWidgets 3.0.x "stable" branch
(but 3.1.x "development" branch is not recommended)
<li>gettext
</ul>

<p>
wxWidgets は NetBSD(pkgsrc) なら pkgsrc/x11/wxGTK30 です。
OPTIONS:gtk3 で動作確認しています。
<q>If you use NetBSD(pkgsrc),
wxWidgets is pkgsrc/x11/wxGTK30 (OPTIONS:gtk3 is tested).</q>

<p>
(NetBSD 以外でのビルドはサポートしていませんが)
Ubuntu 20.04 ではたぶん以下のパッケージが必要です。
<q>You may need the following packages on Ubuntu 20.04
(though we won't support non-NetBSD platform).</q>
</p>
<ul>
<li>
bmake
build-essential
gettext
libbsd-dev
libedit-dev
libkqueue-dev
libwxgtk3.0-gtk-dev
zlib1g-dev
</ul>

<p>
nono のソースアーカイブを展開したら以下のようにビルドします。
configure には環境変数 CC、CXX でコンパイラを指定することが出来ます。
また wx-config が標準的な名前で提供されていないために見付けられない場合には
環境変数 WX_CONFIG にパスを指定することが出来ます。
<q>Extract the nono's source archive and build as following.
You can specify C/C++ compiler using environment variable CC and CXX
if configure cannot find standard name suitable compiler.
Also, you can specify wx-config path using environment variable WX_CONFIG
if configure cannot find wx-config.</q>
</p><p>
<tt>-DNOTEST</tt> は開発者用のテストツールをビルドしないためのものです。
<q>With <tt>-DNOTEST</tt>, the build excludes test tools which is needed
for developers only.</q>

<blockquote class="cons"><pre>
% ./configure
% make -DNOTEST depend
% make -DNOTEST
% su
# make install
</pre></blockquote>

<p>
2つの実行ファイルがインストールされます。
<tt>nono</tt> が GUI 版実行ファイル、
<tt>nono-cli</tt> がコマンドライン版です。
<q>Two executables will be installed.
<tt>nono</tt> is the GUI executable and <tt>nono-cli</tt> is
the command line executable.</q>

</div>


<a name="execute"></a>
<h4>2. 実行方法 <q>How to execute</q></h4>
<div class="main">
nono は複数機種に対応しているため、設定なしでは起動できません。
デフォルトではカレントディレクトリの nono.cfg という設定ファイルを読みます。
<tt>-c</tt> オプションで設定ファイル (VM ディレクトリ) の位置を指定したり、
あるいは <tt>-V</tt> オプションで設定を列挙することも出来ます。
<q>
nono supports multiple architectures so that
it needs a configuration file.
By default, nono reads a configuration file names nono.cfg in
the current directory.
You can specify the configuration file path (or VM directory) using
<tt>-c</tt> option, or enumerate the configuration parameters using
<tt>-V</tt> option.</q>
</div>

<a name="commandline"></a>
<h5>2.1. コマンドラインオプション <q>Command Line Option</q></h5>
<div class="main">
<dl>
<?php
function item($name, $desc)
{
	if (!is_array($name)) {
		$name = array($name);
	}
	foreach ($name as $n) {
		print "<dt class=dt-indent><tt>{$n}</tt></dt>\n";
	}
	print "<dd>{$desc}</p></dd>\n";
}
function option_item($name, $desc)
{
	item($name, $desc);
}

option_item("-c <i>vmpath</i>", <<<_EOM_
VM ディレクトリ/設定ファイルを指定します。
<i>vmpath</i> がディレクトリならそのディレクトリの中の nono.cfg
を設定ファイルとします。
<i>vmpath</i> がファイルならそれを設定ファイルとします。
そしていずれの場合も
設定ファイルがあるディレクトリを VM ディレクトリとします。
-c オプションを省略すると <i>vmpath</i> をカレントディレクトリとします。
<q>Specifies the VM directory/configuration file.
If <i>vmpath</i> is a directory,
make nono.cfg in that directory a configuration file.
Or if <i>vmpath</i> is a file,
make the specified file a configuration file.
And in both cases,
make the directory where that file is located a VM directory.
If <tt>-c</tt> option is omitted,
<i>vmpath</i> is considered as the current directory.</q>
_EOM_
);

option_item("-f", <<<_EOM_
高速モードで起動します。
GUI なら起動後にもメニューから変更できますが、その初期値を変えるだけです。
<q>Boot as the fast mode.
You can change this mode on GUI menu after boot,
and the option only changes its initial state.</q>
_EOM_
);

option_item("--fontsize <i>height</i>", <<<_EOM_
GUI 版のみ。
全サブウインドウの起動時のフォントサイズを指定します。
<tt><i>height</i></tt> には 12, 16
のいずれかを指定します。デフォルトは <tt>12</tt> です。
起動後にメニューから変更できます。
<q>
GUI Only.
Specifies the initial fontsize on all sub windows.
<tt><i>height</i></tt> is one of 12 or 16.
The default value is <tt>12</tt>.
You can change this value on GUI menu after boot.</q>
_EOM_
);

option_item(array("-s <i>scale</i>",
	"--scale <i>scale</i>"), <<<_EOM_
GUI 版のみ。
メインウィンドウの起動時のスケールを実数で指定します。
起動後にもメニューからプリセットされた倍率には変更可能ですが、
任意倍率は起動時のみ指定可能です
<span class=strike>(そのうちなんとかしたい)</span>。
デフォルトは 1.0 です。
<q>
GUI Only.
Specifies the initial main window scale in real number.
You can change this scale on GUI menu after boot,
but unlike this option, there are only a few preset choices
<span class=strike>(Should be improved in someday)</span>.
The default value is 1.0</q>
_EOM_
);

option_item("--show-config", <<<_EOM_
設定ファイルと <tt>-V</tt> オプションを読み込んだ結果を表示します。
<q>Shows the result of reading configuration file and
parsing <tt>-V</tt> options.</q>
_EOM_
);

option_item("-v", <<<_EOM_
バージョンを表示します。
<q>Shows the version.</q>
_EOM_
);

option_item("-V <i>name</i>=<i>value</i>", <<<_EOM_
設定ファイルで指定した <tt><i>name</i>=<i>configvalue</i></tt> の代わりに
このオプションの <tt><i>name</i>=<i>value</i></tt> を適用します。
<tt><i>name</i></tt> が正しくない場合はエラー終了します。
<q>Use this <i>name</i>=<i>value</i>
instead of <i>name</i>=<i>configvalue</i> specified in configuration file.
If <tt><i>name</i></tt> is not correct, it will exit on error.</q>
_EOM_
);

option_item("-X <i>file</i>", <<<_EOM_
ホストの <tt><i>file</i></tt> をロードして実行します。
<tt><i>file</i></tt> が相対パスの場合カレントディレクトリからのパスになります。
ファイルが gzip 圧縮されていれば自動的に展開します。
(展開後の) ファイル形式は a.out (OMAGIC) か ELF で、
実際にはブートローダとカーネル程度しか想定していません。
設定の <tt>prom-image</tt>、<tt>luna-dipsw1</tt> の内容(後述)によらず
内蔵の互換 PROM で起動し、直ちにホストファイルを実行します。
<q>Loads and executes host's <tt><i>file</i></tt>.
If <tt><i>file</i></tt> is relative path,
it is path from the current directory.
If the file is gzip'd, it is automatically extracted.
The supported file format (after extracting) is a.out (OMAGIC) or ELF.
Actually, it only assumes bootloaders or kernels.
Regardless of <tt>prom-image</tt> and <tt>luna-dipsw1</tt> configuration
(see below),
the internal emulated PROM loads and executes the host file immediately.</q>
<p>
現在のところ以下を起動できますが、
いずれも起動元デバイスが取得できないなどの問題はあるかも知れません。
<q>Now the following can be booted.
Note that it may not obtain some information that where did I boot from,
for example.</q>
<ul>
<li>NetBSD/luna68k /boot and kernels.
<li>NetBSD/x68k /boot and kernels.
<li>OpenBSD/luna88k /boot and kernels.
<li>m68k ELF object (Experimental)
<li>Human68k .x executable (Experimental)
</ul>
_EOM_
);
?>
</dl>

以下開発用。<q>For developers:</q>
<dl>
<?php
// 項目間を空けない
function option_dev($name, $desc)
{
	print "<dt class=dt-indent><tt>{$name}</tt></dt>\n";
	print "<dd>{$desc}</dd>\n";
}

option_dev("-b <i>hexaddr</i>[,<i>skipcount</i>]", <<<_EOM_
デバッガのブレークポイントを 16進数で指定します。
_EOM_
);

option_dev("-C", <<<_EOM_
ログをコンソールにも出力します。
通常はログウィンドウにだけ出力されます。
_EOM_
);

option_dev("-d", <<<_EOM_
起動時にデバッガプロンプトで停止します。
_EOM_
);

option_dev("-D", <<<_EOM_
コンソールをデバッガとして使用します。
過去との互換性のために存在していますが、
<tt>-V debugger-driver=stdio</tt> と等価です。
_EOM_
);

option_dev("-L <i>name1</i>=<i>level1</i>[,<i>name2</i>=<i>level2</i>,...]",
<<<_EOM_
ログレベルを指定します。
カンマで区切って複数指定することも出来ます。
<tt>-Lhelp</tt> で name の一覧を表示します。
_EOM_
);

option_dev("--load-only <i>file</i>", <<<_EOM_
<tt>-X</tt> オプションと同様ですが、PROM のプロンプトで停止します。
_EOM_
);

option_dev("-M <i>name</i>[,<i>name2</i>,...]", <<<_EOM_
起動時に表示するモニタウィンドウを指定します。
カンマで区切って複数指定することも出来ます。
<tt>-Mhelp</tt> で name の一覧を表示します。
_EOM_
);


?>
</dl>
</div>


<a name="configuration"></a>
<h4>3. 設定 <q>Configuration</q></h4>
<div class="main">
VM の設定ファイルはその VM ディレクトリ内の nono.cfg
(または -c で指定したファイル) です。
書式は <tt>key = value</tt> 形式で1行1項目ずつです。
<tt>key</tt> と <tt>value</tt> の前後の空白は取り除かれます。
また空行と "<tt>#</tt>" で始まる行は無視します。
知らないキーは警告を出した上で無視します。
<q>VM configuration file is nono.cfg
(or the file specified by -c option) in the VM directory.
Its syntax is <tt>key = value</tt> format, one per line.
White spaces before and after <tt>key</tt> and <tt>value</tt> are ignored.
Blank lines, lines beginning with "<tt>#</tt>" are also ignored.
The lines with unrecognized key are ignored with a warning.</q>
<p>
設定項目は次の通りです。
<q>The configuration items are:</q>
<dl>
<?php
function config_item($name, $desc)
{
	item($name, $desc);
}

config_item("vmtype = <i>string</i>", <<<_EOM_
VM 種別を以下のいずれかから指定します。
省略不可です。
<q>Specifies the VM type from the following.
This field is mandatory.</q>
<table cellspacing=0 cellpadding=0>
<tr><td>&nbsp;<td><tt>luna</tt>	<td>… LUNA-I
<tr><td><td><tt>luna88k</tt>	<td>… LUNA-88K
<tr><td><td><tt>x68030</tt>		<td>… X68030 (Experimental)
</table>
_EOM_
);

config_item("cgrom-image = <i>path</i>", <<<_EOM_
(X68030 Only)
X68030 の外部 CGROM イメージファイルのパスを指定します。
CGROM は 768KB です。
<i>path</i> がファイル名のみなら VM ディレクトリとその親ディレクトリからこのファイル名を検索します。
<i>path</i> が相対パスなら VM ディレクトリからの相対パスになります (現在のディレクトリからではありません)。
空にすると nono 内蔵の互換 CGROM を使用します。
デフォルトは空です。
<q>Specifies the X68030's external CGROM image file path.
This CGROM is 768KB.
If the <i>path</i> does not have any path delimiters,
the VM directory and then its parent directory will be searched.
If the <i>path</i> is a relative path,
it will be path from the VM directory, not from the current directory.
If the <i>path</i> is empty, nono's builtin compatible CGROM will be used.
The default value is empty.</q>
_EOM_
);

config_item("clock-sync = <i>value</i>", <<<_EOM_
仮想マシン内の時刻の同期方法を指定します。
<tt>real</tt> なら実時間に同期、<tt>virtual</tt> なら仮想時間に同期します。
デフォルトは <tt>real</tt> です。
この機能は実験中のため将来予告なく仕様が変更になる可能性があります。
<q>Specify how to synchronize the time in virtual machine.
If <tt>real</tt>, synchronize with the real time;
if <tt>virtual</tt>, synchronize with the virtual time.
The default is <tt>real</tt>.
This feature is under experimentation and may be changed in the future
without notice.</q>
_EOM_
);

config_item("debugger-driver = <i>string</i>", <<<_EOM_
デバッガのコンソールドライバを指定します。
<tt>stdio</tt>、<tt>tcp</tt>、<tt>none</tt> が選択可能です。
<tt>stdio</tt> は標準入出力を使用します。
<tt>tcp</tt> は TCP ポートで TELNET プロトコルで待ち受けます。
<tt>none</tt> ならホスト側とは一切通信を行いません。
デフォルトは <tt>none</tt> です。
<q>Specifies console driver of the debugger.
<tt>stdio</tt>, <tt>tcp</tt>, and <tt>none</tt> can be specified.
<tt>stdio</tt> uses the standard input/output.
<tt>tcp</tt> listens on TCP port using TELNET protocol.
<tt>none</tt> doesn't make any communication with the host.
The default is <tt>none</tt>.</q>
_EOM_
);

config_item("debugger-tcp-port = <i>integer</i>",
<<<_EOM_
デバッガのコンソールドライバが tcp の時の TCP 待ち受けポート番号を指定します。
<q>Specify the TCP port number that debugger console driver listens.</q>
_EOM_
);

config_item("ethernet-macaddr = <i>string</i>", <<<_EOM_
(LUNA Only)
イーサネットデバイスの仮想マシン側の MAC アドレスを指定します。
<tt><i>XX</i>:<i>XX</i>:<i>XX</i>:<i>XX</i>:<i>XX</i>:<i>XX</i></tt>
形式で指定します。
<tt>auto</tt> なら自動的に決定します。
デフォルトは <tt>auto</tt> です。
_EOM_
);

config_item("fd-drive = <i>integer</i>", <<<_EOM_
(X68030 Only)
フロッピードライブの数を 0 から 4 で指定します。
デフォルトは 2 です。
<q>Specify the number of floppy drives from <tt>0</tt> to <tt>4</tt>.
The default is <tt>2</tt>.</q>
_EOM_
);

config_item("fd<i>N</i>-image = <i>path</i>", <<<_EOM_
(X68030 Only)
フロッピードライブ <i>N</i> に起動時に挿入するディスクイメージを指定します。
イメージパスが相対パスなら VM ディレクトリからの相対パスになります。
起動後はメニューから操作できます。
デフォルトは空です。
<q>Specify a diskimage to be inserted to floppy drive <i>N</i> at startup.
If the <i>path</i> is relative path, it is from the VM directory.
You can also operate them from menu after startup.
The default is empty.</q>
_EOM_
);

config_item("fpu-type = <i>value</i>", <<<_EOM_
(X68030 Only)
FPU を装着するかどうか指定しています。
<tt>none</tt> なら FPU を装着しません。
<tt>68881</tt> なら FPU として 68881 を装着します。
現状 68882 はサポートしていません。
また <tt>none</tt> の代わりに <tt>0</tt>、
<tt>68881</tt> の代わりに <tt>1</tt> と書くこともできます。
デフォルトは <tt>68881</tt> です。
LUNA-I はたぶん 68881 搭載モデルのみのため設定できません。
<q>Specify whether to install FPU or not.
If <tt>none</tt>, FPU is not installed.
If <tt>68881</tt>, 68881 FPU is installed.
68882 is not supported yet.
And, <tt>none</tt> can also be written as <tt>0</tt>,
<tt>68881</tt> can also be written as <tt>1</tt>.
The default is <tt>68881</tt>.
This item cannot be specified on LUNA-I
because (probably) all LUNA-I has 68881.</q>
_EOM_
);

config_item("hostcom-driver = <i>string</i>", <<<_EOM_
シリアルポートのホスト側ドライバを指定します。
<tt>stdio</tt>、<tt>tcp</tt>、<tt>none</tt> が選択可能です。
<tt>stdio</tt> は標準入出力を使用します。
<tt>tcp</tt> は TCP ポートで TELNET プロトコルで待ち受けます。
<tt>none</tt> ならホスト側とは一切通信を行いません。
デフォルトは <tt>none</tt> です。
<q>Specifies the host driver of serial port.
<tt>stdio</tt>, <tt>tcp</tt>, and <tt>none</tt> can be specified.
<tt>stdio</tt> uses the standard input/output.
<tt>tcp</tt> listens on TCP port using TELNET protocol.
<tt>none</tt> doesn't make any communication with the host.
The default is <tt>none</tt>.</q>
_EOM_
);

config_item("hostcom-tcp-port = <i>integer</i>", <<<_EOM_
ホストドライバが tcp の時の TCP 待ち受けポート番号を指定します。
<q>Specify the TCP port number that host driver listens.</q>
_EOM_
);

config_item("hostcom-fallback = <i>integer</i>", <<<_EOM_
<tt>hostcom-driver</tt> で指定したドライバが使用可能でなかった時、
0 ならプロセスを終了します。1 なら <tt>none</tt> を選択して実行を継続します。
デフォルトは 0 です。
<q>Specify the behavior when the driver which is selected by
<tt>hostcom-driver</tt> is unusable;
terminate the process if 0,
or continue to run using the <tt>none</tt> driver if 1.
The default is 0.</q>
_EOM_
);

config_item("hostkbd-input = <i>string</i>", <<<_EOM_
ホストキーボードの入力モードを指定します。
<tt>char</tt> ならキャラクタ入力モード、
<tt>jp</tt> なら日本語キーボードモードです。
デフォルトは <tt>char</tt> です。
詳細は <a href="#aboutvm">VM について</a> の章を参照してください。
<q>Specifies the input mode of the host keyboard.
<tt>char</tt> means the character mode,
<tt>jp</tt> means the Japanese keyboard mode.
The defualt is <tt>char</tt>.
See <a href="#aboutvm">About VM</a> for details.</q>
_EOM_
);

config_item("hostnet-driver = <i>string</i>", <<<_EOM_
(LUNA Only)
イーサネットデバイスのホスト側ドライバを指定します。
<tt>afpacket</tt>、<tt>bpf</tt>、<tt>tap</tt> のうち
ホスト OS がサポートしているものと <tt>auto</tt>、<tt>none</tt> が選択可能です。
<tt>afpacket</tt> は Linux の AF_PACKET ソケットを使用します。
<tt>bpf</tt> は bpf(4) デバイスを使用します。
<tt>tap</tt> は tap(4) インタフェースを使用します。
<tt>none</tt> ならホスト側とは一切通信を行いません。
<tt>auto</tt> ならホスト OS がサポートしているもののうち
<tt>tap</tt> → <tt>afpacket</tt> → <tt>bpf</tt> を順に試します。
デフォルトは <tt>auto</tt> です。
<q>Specifies the host driver of ethernet device.
<tt>afpacket</tt>, <tt>bpf</tt>, and <tt>tap</tt> can be specified
only if the host OS supports them.
<tt>auto</tt> and <tt>none</tt> can always be specified.
<tt>afpacket</tt> uses Linux's AF_PACKET socket,
<tt>bpf</tt> uses bpf(4) device,
and <tt>tap</tt> uses tap(4) interface.
<tt>none</tt> doesn't make any communication with the host.
If <tt>auto</tt> is specified,
it tries <tt>tap</tt>, <tt>afpacket</tt>, and <tt>bpf</tt> in that order
(if the host OS supports them).
The default is <tt>auto</tt>.</q>
_EOM_
);

config_item("hostnet-afpacket-ifname = <i>ifname</i>", <<<_EOM_
(LUNA Only)
ホストドライバが afpacket の時にバインドするインタフェースを1つ指定します。
<i>ifname</i> が <tt>auto</tt>
なら使用可能なインタフェースを1つ自動的に選択します。
デフォルトは <tt>auto</tt> です。
なお、このホストドライバではローカルホストとの通信は出来ません。
<q>Specify an interface name to bind to, when the host driver is afpacket.
If <tt>auto</tt> is specified as <i>ifname</i>,
it selects an usable interface automatically.
The default is <tt>auto</tt>.
Note that this host driver cannot communicate with the localhost.</q>
_EOM_
);

config_item("hostnet-bpf-ifname = <i>ifname</i>", <<<_EOM_
(LUNA Only)
ホストドライバが bpf の時にバインドするインタフェースを1つ指定します。
<i>ifname</i> が <tt>auto</tt>
なら使用可能なインタフェースを1つ自動的に選択します。
デフォルトは <tt>auto</tt> です。
なお、このホストドライバではローカルホストとの通信は出来ません。
<q>Specify an interface name to bind to, when the host driver is bpf.
If <tt>auto</tt> is specified as <i>ifname</i>,
it selects an usable interface automatically.
The default is <tt>auto</tt>.
Note that this host driver cannot communicate with the localhost.</q>
_EOM_
);

config_item("hostnet-tap-devpath = <i>path</i>", <<<_EOM_
(LUNA Only)
ホストドライバが tap の時に使用するデバイスを1つフルパスで指定します。
デフォルトは <tt>auto</tt> で、この場合は自動的にデバイスを選択します。
この時の探し方はホスト OS によって異なります。
Linux なら <tt>/dev/net/tun</tt> を指定したのと同じです。
OpenBSD なら <tt>/dev/tap0</tt> から <tt>/dev/tap9</tt> まで
順番にオープンできるまで試します。
NetBSD (と FreeBSD) ならまず <tt>/dev/tap</tt> でクローニングを試み、
それが失敗すれば <tt>/dev/tap0</tt> から <tt>/dev/tap9</tt> までを
順番にオープンできるまで試します。
いずれの場合も VM ディレクトリかその親ディレクトリに
<tt>nono-ifup</tt>, <tt>nono-ifdown</tt> という名前のシェルスクリプトが必要です。
何もすることがない場合でも正常終了する空のシェルスクリプトを用意してください。
<q>
Specify a device pathname, when the host driver is tap.
The default is <tt>auto</tt>.
The behavior in this case depends on the host OS.
On Linux, it's the same as <tt>/dev/net/tun</tt>.
On OpenBSD, it will try from <tt>/dev/tap0</tt> to <tt>/dev/tap9</tt>
until successful.
On NetBSD (and FreeBSD), it will try cloning by <tt>/dev/tap</tt> first.
If that fails, then try from <tt>/dev/tap0</tt> to <tt>/dev/tap9</tt>
until successful.
In all cases, you need to prepare two shell script files which names are
<tt>nono-ifup</tt> and <tt>nono-ifdown</tt> in the VM directory or
its parent directory.
Even if you don't have anything to do in these scripts,
you need to prepare empty scripts that will terminate successfully.</q>
_EOM_
);

config_item("hostnet-fallback = <i>integer</i>", <<<_EOM_
(LUNA Only)
<tt>hostnet-driver</tt> で指定したドライバが使用可能でなかった時、
0 ならプロセスを終了します。
1 なら none を選択して実行を継続します。
デフォルトは 0 です。
<q>Specify the behavior when the driver which is selected by
<tt>hostnet-driver</tt> is unusable;
terminate the process if 0,
or continue to run using the <tt>none</tt> driver if 1.
The default is 0.</q>
_EOM_
);

config_item("iplrom1-image = <i>path</i>", <<<_EOM_
(X68030 Only)
X68030 の外部 IPLROM イメージファイルのパスを指定します。
こちらは 0xfe0000..0xffffff の 128KB の部分で、IPLROM30 と呼ばれているほうです。
<i>path</i> がファイル名のみなら VM ディレクトリとその親ディレクトリからこのファイル名を検索します。
<i>path</i> が相対パスなら VM ディレクトリからの相対パスになります (現在のディレクトリからではありません)。
空にすると無償配布されている IPLROM30 を使用します。
デフォルトは空です。
<q>Specifies the X68030's external IPLROM image file path.
This one contains 128KB at 0xfe0000..0xffffff and is known as IPLROM30.
If the <i>path</i> does not have any path delimiters,
the VM directory and then its parent directory will be searched.
If the <i>path</i> is a relative path,
it will be path from the VM directory, not from the current directory.
If the <i>path</i> is empty, the public released IPLROM30 will be used.
The default value is empty.</q>
_EOM_
);

config_item("iplrom2-image = <i>path</i>", <<<_EOM_
(X68030 Only)
X68030 の外部 IPLROM イメージファイルのパスを指定します。
こちらは 0xfc0000..0xfdffff の 128KB の部分で、ROM30 と呼ばれているほうです。
<i>path</i> がファイル名のみなら VM ディレクトリとその親ディレクトリからこのファイル名を検索します。
<i>path</i> が相対パスなら VM ディレクトリからの相対パスになります (現在のディレクトリからではありません)。
空にすると内蔵 ROM を使用します。
内蔵 ROM は NetBSD/x68k が起動する程度の SCSI IOCS のみサポートしています。
デフォルトは空です。
<q>Specifies the X68030's external IPLROM image file path.
This one contains 128KB at 0xfc0000..0xfdffff and is known as ROM30.
If the <i>path</i> does not have any path delimiters,
the VM directory and then its parent directory will be searched.
If the <i>path</i> is a relative path,
it will be path from the VM directory, not from the current directory.
If the <i>path</i> is empty, internal emulated ROM will be used.
The ROM supports only enough SCSI IOCS to boot NetBSD/x68k.
The default value is empty.</q>
_EOM_
);

config_item("keyboard-connect = <i>integer</i>", <<<_EOM_
起動時にキーボードを本体に接続するかどうかを指定します。
1 なら接続し、0 なら接続しません。デフォルトは 1 です。
起動後はメニューから変更可能です。
<q>Specify whether to connect keyboard on boot.
If 1, it is connected; if 0, it isn't connected.  The default is 1.
You can change it on GUI menu after boot.</q>
_EOM_
);

config_item("luna-adjust-misused-epoch = <i>integer</i>", <<<_EOM_
(LUNA Only)
LUNA で誤った RTC epoch を採用している OS 向けに RTC
エミュレーションを補正するかどうかを
指定します。0 なら補正をしません(実機と同じ動作)、
1 なら補正します(現実世界と同じ動作)。
デフォルトは 1 で、通常 1 のままで使用して問題ありません。
<q>
Specifies whether nono corrects RTC emulation for OSes
that adopts wrong RTC epoch on LUNA.
0 means making no correction
(this is the same behavior as the actual machine).
1 means making correction
(this is the same behavior as the real world).
The default is 1.  Normally, leave it 1.</q>
<p>
LUNA で採用している RTC (MK48T02) は2桁で保持している年の値が
4 で割り切れる年をうるう年とする仕様です。
ところが NetBSD/luna68k、OpenBSD/luna88k などはこの
RTC の年の値を 1970年からの経過年として使用しています。
例えば1970年はうるう年ではないため 2月28日の翌日は 3月1日ですが、
MK48T02 的には 00年であるためうるう年と認識し 2月28日の翌日が2月29日になります。
このように、これらの OS を使っている場合実機の
RTC は4年のうち約2年間、1日ずれた日付を指しているようです。
しかしながら、RTC の時刻は OS 起動時に一度読んだ後は基本的に参照しない上、
今時必要なら NTP で時間を合わせるため、
実機でも問題が顕在化することはまずないと思います。
nono の場合は実機と異なり、アプリケーション実行中しか RTC が進まないため、
補正がない場合の動作が問題になるのは
nono を起動したまま偶数年の2月末日から日付をまたいで、
かつ nono を起動したまま OS を再起動して NTP などで時刻修正を行わなかった時
だけだと思います。
このオプションはほぼ開発者向けの動作確認用です。
_EOM_
);

config_item("luna-dipsw1 = <i>string</i>", <<<_EOM_
(LUNA Only)
本体前面 DIPSW#1-1..#1-8 の内容を指定します。
"<tt>0</tt>" を DOWN、"<tt>1</tt>" を UP として、
これを8つ並べた形式で、前から順に #1..#8 に対応します。
<q>Specifies status of the front panel DIPSW#1-1..#1-8 using 8 digit.
"<tt>0</tt>" means DOWN and "<tt>1</tt>" means UP.
The first character corresponds to #1 and
the eighth character corresponds to #8.</q>
<p>
LUNA-I でのデフォルトは <tt>11110111</tt> です。
各スイッチの内容は以下のリンクを参照してください。
<q>On LUNA-I, the default value is <tt>11110111</tt>.
See the following link about DIPSW.</q><br>
→ <a href="https://wiki.netbsd.org/ports/luna68k/luna68k_info/"
>NetBSD/luna68k: Information</a>
<br>
LUNA-88K でのデフォルトは <tt>11111111</tt> です。
各スイッチの内容は以下のリンクを参照してください。
<q>On LUNA-88K, the default value is <tt>11111111</tt>.
See the following link about DIPSW.</q><br>
→ <a href="http://man.openbsd.org/boot_luna88k.8"
>OpenBSD manual pages: boot_luna88k(8)</a>
_EOM_
);

config_item("luna-dipsw2 = <i>string</i>", <<<_EOM_
(LUNA Only)
本体前面 DIPSW#2-1..#2-8 の内容を指定します。
書式は <tt>luna-dipsw1</tt> と同じです。
デフォルトは <tt>11111111</tt> です。
<q>Specifies status of the front panel DIPSW#2-1..#2-8.
The same syntax as <tt>luna-dipsw1</tt> is used.
The default value is <tt>11111111</tt>.</q>
<p>
NetBSD/luna68k のブートローダは、
DIPSW#2 が "<tt>11111111</tt>" なら自動的にカーネルをロードして実行し、
どれかでも "<tt>0</tt>" にするとプロンプトで停止するようです。
<span class=strike>(本当は #8 だけで制御するつもりだったんじゃないかという気がします)</span>
<q>NetBSD/luna68k bootloader will automatically load and execute the kernel,
if the DIPSW#2 is "<tt>11111111</tt>".
Otherwise, the bootloader will enter interactive mode.
<span class=strike>(I doubt that they actually wanted to switch with only #8)
</span></q>
_EOM_
);

config_item("luna-video-plane = <i>integer</i>", <<<_EOM_
(LUNA Only)
LUNA のビデオボードのプレーン数を 1 か 4 で指定します。
1 ならモノクロビデオボード、
4 なら16色 (4bpp) ビデオボードに相当します。
8bpp ボードは(まだ?)サポートしていません。
デフォルトは今の所 1 です (将来変更するかもしれません)。
<q>Specify number of planes on LUNA video board.
The valid values are 1 or 4.
1 means a monochrome video board,
4 means 16-color (4bpp) video board.
Note that 8bpp video board is not (yet?) supported.
The default value is 1 for now (It may be changed in the future).</q>
_EOM_
);

config_item("monitor-rate = <i>integer</i>", <<<_EOM_
テキスト系モニタウィンドウの更新頻度を Hz 単位で指定します。
1 から 60 までの間で指定でき、デフォルトは 20Hz です。
起動後にメニューからプリセットされた頻度には変更可能です。
<q>Specifies refresh rate of all text monitor windows in Hz.
It ranges from 1 to 60.  The default is 20Hz.
You can change this value on GUI menu after boot,
but unlike this configuration value, there are only a few preset choices.</q>
_EOM_
);

config_item("mpu-clock = <i>value</i>", <<<_EOM_
MPU のクロック数を MHz 単位で指定します。
デフォルトは LUNA-I なら 20MHz、LUNA-88K と X68030 なら 25MHz です。
<q>Specifies the MPU clock in MHz.
The default value is 20MHz on LUNA-I, or 25MHz on LUNA-88K and X68030.</q>
_EOM_
);

config_item("mpu-pseudo-stop = <i>integer</i>", <<<_EOM_
(LUNA-88K Only)
m88100 にて疑似 STOP 状態を有効にするかどうかを指定します。
0 なら無効(実機と同じ動作)、1 なら有効で、デフォルトは 1 です。
m88100 には、m68k の STOP 命令 (割り込みが上がるまで何もせず待つ)
に相当する命令がなく、
大抵ビジーウェイトループで割り込みが上がるのを待つことになります。
これは実機では (消費電力を減らす手段がないという些細な問題以外には)
何のデメリットもないのですが、
エミュレータで特に高速動作させている時には割り込みが上がるまで
(例えば人間がキーを入力するまで) ホスト CPU パワーを使い潰してビジーウェイトループを実行し続けることになり、ホスト CPU があっつあつになります。
それを防ぐための機能です。
特徴的な命令列を検出して実現しているので、すべての状況で動作するわけではありません。
_EOM_
);

config_item("prom-image = <i>path</i>", <<<_EOM_
(LUNA Only)
LUNA-I/LUNA-88K の外部 ROM イメージファイルのパスを指定します。
<i>path</i> がファイル名のみなら VM ディレクトリとその親ディレクトリからこのファイル名を検索します。
<i>path</i> が相対パスなら VM ディレクトリからの相対パスになります (現在のディレクトリからではありません)。
空にすると内蔵 ROM を使用します。
デフォルトは空です。
<q>Specifies the LUNA-I/LUNA-88K's external ROM image file path.
If the <i>path</i> does not have any path delimiters,
the VM directory and then its parent directory will be searched.
If the <i>path</i> is a relative path,
it will be path from the VM directory, not from the current directory.
If the <i>path</i> is empty, internal emulated ROM will be used.
The default value is empty.</q>
<p>
実機を持っていない場合はこの値を空に (= デフォルトのままに) しておくと、
nono 内蔵のなんちゃって下位互換 ROM で起動します。
<q>If you does not have the real LUNA machines,
you can boot with nono's internal downward compatible emulated ROM
if you set this field empty (or leave it as the default).</q>
<p>
LUNA-I 実機を持っている場合は
ROM ファイルを指定することで実機 ROM で起動できます。
ROM ファイルは実機の 0x41000000-0x4101ffff (128KB) を保存したものです。
今のところ ROM は V4.22 (Thu Jul 27 11:45:42 1989) のみサポートしています。
それ以外については何も分かりません。
<q>If you have the real LUNA-I machine,
you can boot with the real ROM spcifying the ROM file path.
The ROM file is extracted from 0x41000000-0x4101ffff (128KB) of
the real LUNA-I machine.
For now, only V4.22 (Thu Jul 27 11:45:42 1989) is supported.
I have no idea about other ROMs.</q>
<p>
LUNA-88K 実機の場合は 0x41000000-0x4103ffff (256KB) を保存したものです。
今のところ ROM は version 1.20 のみサポートしています。
<q>
For LUNA-88K,
the ROM file is extracted from 0x41000000-0x4103ffff (256KB).
For now, only version 1.20 is supported.</q>
_EOM_
);

config_item("ram-size = <i>integer</i>", <<<_EOM_
搭載する RAM サイズを MB 単位で指定します。<q>Specifies the RAM size in MB.</q>
<ul>
<li>LUNA-I のデフォルトは 16MB です。
16MB 未満は 4MB 単位で、
16MB 以上は 255MB まで 1MB 単位で指定できます
(ちなみに NetBSD/luna68k の起動には最低でも 8MB 必要です)。
<q>On LUNA-I, the default is 16MB.
If the size is less than 16MB, you can specify in 4MB unit.
If larger, you can specify up to 255MB in 1MB unit.
By the way, NetBSD/luna68k needs at least 8MB to boot.</q>
<li>LUNA-88K のデフォルトは 64MB です。
64MB 未満は 16MB 単位で、
64MB 以上は暫定で 255MB まで 1MB 単位で指定できます。
<q>On LUNA-88K, the default is 64MB.
If the size is less than 64MB, you can specify in 16MB unit.
If larger, you can specify up to tentative 255MB in 1MB unit.</q>
<li>X68030 のデフォルトは 12MB です。
4MB から 12MB まで 1MB 単位で指定できます。
<q>On X68030, the default is 12MB.
You can specify it in 1MB unit from 4MB to 12MB.</q>
</ul>
_EOM_
);

config_item("rtc-epoch-year = <i>integer</i>", <<<_EOM_
(LUNA Only)
RTC (MK48T02) の基準年を指定します。
デフォルトは 1970年です。
NetBSD/luna68k、OpenBSD/luna88k はいずれも基準年を 1970年としていますので、
デフォルトのままで構いません。
ゲスト OS に 4.4BSD を使用する際は 1900 を指定します。
<q>Specifies the RTC (MK48T02) epoch year.
The default is 1970.
Since both NetBSD/luna68k and OpenBSD/luna88k use 1970 as epoch year,
there is no need to touch this value.
If you boot 4.4BSD on LUNA-I as the guest OS, specify 1900.</q>
_EOM_
);

config_item("show-statuspanel = <i>integer</i>", <<<_EOM_
ステータスパネルを表示するかどうかを指定します。
0 なら非表示、1 なら表示です。
起動後はメニューから変更可能です。
<q>Specifies whether to display the status panel or not.
If 0, it is hidden; if 1, it is shown.
You can change it on GUI menu after boot.</q>
_EOM_
);

config_item("spc0-id<i>N</i>-image = <i>devtype</i>[,<i>path</i>]", <<<_EOM_
SCSI デバイスとイメージを指定します。キーの <i>N</i> には 0 から 7 が入ります。
ただし ID 7 は本体が使用しますので指定しないでください。
値はデバイス種別 <i>devtype</i> とディスクイメージパス <i>path</i>
を "<tt>,</tt>"(カンマ) で区切って並べた形式です。
デバイス種別 <i>devtype</i> は以下のいずれかです。
<q>
Specifies SCSI device and image.  <i>N</i> in the key is 0 to 7.
But don't specify ID 7 because the host uses it.
The value is in a form of device type <i>devtype</i> and
the disk image path <i>path</i> separated by "<tt>,</tt>"(comma).
<i>devtype</i> can be one of the following:</q>
<ul>
<li><tt>hd</tt> … HD drive
<li><tt>cd</tt> … CD-ROM drive
<li><tt>mo</tt> … MO drive
</ul>

<p>
<i>devtype</i> が <tt>hd</tt> なら <i>path</i> は省略できません。
<i>devtype</i> が <tt>cd</tt> か <tt>mo</tt> なら <i>path</i> は省略可能です。
イメージパスが相対パスなら VM ディレクトリからの相対パスになります。
<q>
If <i>devtype</i> is <tt>hd</tt>, <i>path</i> cannot be ommitted.
If <i>devtype</i> is <tt>cd</tt> or <tt>mo</tt>, <i>path</i> can be ommitted.
If the <i>path</i> is relative path, it is from the VM directory.</q>

<p>
例えば、nono.cfg と同じディレクトリに置いた sd0.img を
起動 HDD ディスクイメージとして使い
(LUNA では通常 ID 6 をプライマリ HDD に割り当てます)、
ID 5 に同じディレクトリの install.iso をセットした CD ドライブを、
ID 4 に起動時メディアなしの MO ドライブを接続する場合は次のようになります。
<q>For example, if you use a harddisk image sd0.img placed in the same
directory as nono.cfg
(LUNA usually assigns ID 6 to the primary HDD),
ID 5 for CD-ROM drive that loads install.iso in the same directory,
and ID 4 for MO drive without media on boot,
write as following:</q>
<blockquote><pre>
spc0-id6-image = hd,sd0.img
spc0-id5-image = cd,install.iso
spc0-id4-image = mo
</pre></blockquote>
_EOM_
);

config_item("spc0-id<i>N</i>-seektime = <i>integer</i>", <<<_EOM_
指定の SCSI HDD の平均シークタイムを msec 単位で指定します。
現在のデフォルトは <tt>0</tt> です (S・S・D!! S・S・D!!)。
16 程度を指定すると幾分往時に思いを馳せることが出来るかもしれませんが、
今の所あまり安定していません。
<q>Specifies the average seek time of specified SCSI HDD in msec.
Currently, the default value is <tt>0</tt>
(This may be something like SSD :-).
If you specify about 16 or so, you can feel nostalgic,
but this feature is still unstable.</q>
_EOM_
);

config_item("spc0-id<i>N</i>-writeignore = <i>integer</i>", <<<_EOM_
指定の SCSI HD デバイスへの書き込みを無視するかどうか指定します。
<tt>0</tt> なら通常動作(書き込みを行う)です。
<tt>1</tt> なら書き込みコマンドは成功したように振る舞いますが実際には
ディスクイメージに一切書き戻しません。
fsck を気にせずカーネルのデバッグとかを行いたい場合にはどうぞ。
何が起きるか意味が分からない人は指定しないでください。
デフォルトは <tt>0</tt> です。
<q>Specifies whether nono ignores writing to SCSI HD devices.
<tt>0</tt> means normal operation (writes to the devices).
If <tt>1</tt> is specified,
nono will not actually write back to the disk image
even though the write command is succeeded.
nono's SCSI devices acts as
write command is successfully done but it never writes back
to the actual disk image.
This is useful for kernel debugging because it does not require fsck
after the kernel hangs.
But don't use this flag if you don't understand this paragraph.
The default value is <tt>0</tt>.</q>
<p>
ちなみに、メディアを書き込み禁止にしたい場合はこれではなく、
イメージファイルの書き込み権を落としてください。
<q>By the way, if you want to make the media write-protected,
clear the write permission from the image file
(instead of this setting).</q>
_EOM_
);

config_item("spc0-id<i>N</i>-writeprotect = <i>integer</i>", <<<_EOM_
古いオプションです。
代わりに <tt>spc0-id<i>N</i>-writeignore</tt> を使ってください。
このオプションは開発用です。
<q>Obsolete.
Use <tt>spc0-id<i>N</i>-writeignore</tt> instead.
This option is for developers.</q>
_EOM_
);

config_item("sram-sync-ramsize = <i>integer</i>", <<<_EOM_
(X68030 Only)
SRAM の RAM 容量欄を実際の RAM サイズに合わせるかどうかを指定します。
<tt>0</tt> なら何もしません (実機と同じ)。
<tt>1</tt> なら、アプリケーション起動時に
SRAM の \$ed0008.L (RAM 容量) を ram-size の設定値で更新します。
この更新は、
アプリケーション起動時に SRAM マジックが正しく書かれている場合のみ行われます。
デフォルトは <tt>1</tt> です。
<q>Specify whether to synchronize a RAM size field in SRAM
with real RAM size or not.
If <tt>0</tt>, the application will do nothing (as same as the real).
If <tt>1</tt>, the application updates(synchronizes)
\$ed0008.L (RAM size) in SRAM with configuration value of ram-size.
This update is performed once only if
the SRAM magic string is correct at the application startup.
The default is <tt>1</tt>.</q>
_EOM_
);
?>
</dl>
</div>


<a name="aboutvm"></a>
<?php
$section = 4;
$subsection = 1;
function sub()
{
	global $section;
	global $subsection;
	print "${section}.${subsection}.";
	$subsection++;
}
?>
<h4>4. VM について <q>About VM</q></h4>

<h5><?php sub(); ?> 実装状況 <q>Implementation Status</q>
<div class="main">
<table cellpadding=0 cellspacing=0>
<tr><td style="padding-right: 1em">◎	<td>: ほぼ実装済み <q>Mostly implemented</q>
<tr><td>○	<td>: 困らない程度には実装済み <q>Works</q>
<tr><td>△	<td>: とりあえず動いてる程度 <q>Works somehow</q>
<tr><td>×	<td>: 未実装 <q>Not implemented</q>
<tr><td>—	<td>: 実装予定なし <q>Not planned</q>
</table>
<style type="text/css">
.pghd {
	border: 0px none;
	border-bottom: 1px solid black;
	padding-top: 1em;
}
.pgth {
	border-top: 0px none;
	border-left: 1px solid black;
	border-right: 1px solid black;
	border-bottom: 1px solid black;
	padding-right: 1ex;
}
.pgtd {
	border-top: 0px none;
	border-left: 0px none;
	border-right: 1px solid black;
	border-bottom: 1px solid black;
}
</style>
<table cellspacing=0 cellpadding=0>
<?php
	$table = <<<__EOM__
LUNA-I
MPU (68030)		○	キャッシュ未実装 <q>Cache not implemented</q>
FPU (68881)		◎
BT454			◎
CRTC2			○	標準で使用する値のみサポート
Frame buffer	◎
Keyboard/Mouse	◎
SIO				◎
PIO				○
Ethernet (AM7990)	◎	LARF を除く <q>Except LARF</q>
NVRAM&amp;RTC (MK48T02)	◎
Front LCD		◎
SPC				◎
SCSI HD/CD/MO Devices	○
XP とその向こう<wbr><q>XP and beyonds</q>	×
電源 <q>Power Circuit</q>			○

LUNA-88K specific
MPU (88100 part)	○ 	マルチプロセッサ未実装、バグも未実装? <wbr><q>MP (and bugs?) not implemented</q>
MPU (88200 part)	◎
MPU (88110)		—	実装予定なし <q>Not planned</q>
FUSEROM			○

X68030
MPU (68030)		○	キャッシュ未実装、ファンクションコード未実装 <wbr><q>Cache and FC not implemented</q>
MPU (68EC030)	—	実装予定なし <q>Not planned</q>
FPU (68881)		◎	68882 は未実装 <q>68882 not implemented</q>
DMAC			○
CRTC/VC			△	768x512 のテキスト画面のみサポート <wbr><q>Only 768x512 text screen supported</q>
FDC/FDD			○
標準・拡張エリアセット <q>Areaset</q>	×
ADPCM/OPM		×
グラフィック機能全般 <wbr><q>All other graphics</q>	×	当面予定なし <q>Not scheduled</q>
Keyboard		◎	TVコントロールを除く <q>Except TV Control</q>
Mouse			◎	本体接続のみ
MFP				△
PPI				△
Printer			×
RTC				◎	アラームを除く <q>Except Alarm</q>
SPC				◎
SCSI HD/CD/MO Devices	○
SCC				◎
SRAM			◎
システムポート<q>System port</q>	△
テキスト画面<q>Text screen</q>	○
電源 <q>Power curcuit</q>			○
Nereid Ethernet		×
Nereid Bank memory	×
Nereid USB			—	実装予定なし <q>Not planned</q>
拡張メモリ <q>Extended memory</q>		×
__EOM__;
	$table_lines = preg_split("/\n/", $table);
	foreach ($table_lines as $line) {
		if ($line == "") {
			continue;
		} else if (preg_match("/\t/", $line) == false) {
			// table title
			print "<tr><td colspan=3 class=pghd>{$br}{$line}\n";
			$br = "";
		} else {
			list ($h, $m, $d) = preg_split("/\t+/", $line, 3);
			print "<tr><td class=pgth>{$h}\n";
			print " <td class=pgtd style='padding: 0 1ex 0 1ex'>{$m}\n";
			print " <td class=pgtd style='padding-left: 0.5ex'>{$d}\n";
		}
	}
?>
</table>
</div>

<h5><?php sub(); ?> MPU (68030)</h5>
<div class="main">
68030 はキャッシュが未実装です。
これに伴い命令の所要クロック数もまだ正しくなく、
そのためクロック測定などでは正確な値が出ません。
また ATC (アドレス変換キャッシュ) は実際の仕様とは異なり高速化してあります。
<q>68030 cache is not implemented.
Therefore, number of clock cycle is not correct.
ATC (Address Translation Cache) is differ from and faster than the real 68030.</q>
</div>

<h5><?php sub(); ?> LUNA の ROM エミュレーション</h5>
<div class="main">
LUNA-I、LUNA-88K で PROM の外部イメージを指定しない場合
(<tt>prom-image</tt> が空の場合)、
内蔵の独自 ROM が動作し、コンソールモニタが起動します。
このコンソールモニタの動作は実機のそれとよく似ていますが、
これは実機確認の際に操作方法が違いすぎると(自分が)困るため仕方なく似せているもので、
忠実なコピーを作る目的ではありません。
</div>

<h5><?php sub(); ?> X68030 の ROM (ROM30) エミュレーション</h5>
<div class="main">
X68030 で ROM30 の外部イメージを指定しない場合
(<tt>iplrom2-image</tt> が空の場合)、内蔵の独自 ROM が組み込まれます。
本来の ROM30 には SCSIIOCS の他に ROM デバッガなどが含まれていますが、
内蔵 ROM はこのうち SCSIIOCS を NetBSD/x68k が起動するのに十分なだけしか
実装していません。
Human68k を起動する場合はお手元の実機から ROM イメージを作成してください。
</div>

<h5><?php sub(); ?> ステータスパネル <q>Status Panel</q></h5>
<div class="main">
ステータスパネル中央にあるパフォーマンスメータのアイコンは
高速モードの状態を表示しています。
ダブルクリックすると高速/等速モードの指定を切り替えることができます。
<q>The performance meter's icon at the center of the status panel
shows the VM speed status.
You can switch full speed / syncronized mode by double-clicking
on this icon.</q>
<table>
<tr><td valign=top>
<image src="image/sp-ffmark0.png" style="padding:0.5ex; background-color:white">
<td valign=top>…
<td valign=top>
マークなしの場合、ユーザが等速モードを指定していて、等速モードで実行中です。
<q>When no icons are displayed,
the user has specified syncronized mode and
the VM is running in synchronized mode.</q>
<tr><td valign=top>
<image src="image/sp-ffmark2.png" style="padding:0.5ex; background-color:white">
<td valign=top>…
<td valign=top>
三角3つの場合、ユーザが高速モードを指定していて、高速モードで実行中です。
高速モード中はパーセントではなく何倍速で動作しているかを表します。
<q>When an icon (three triangles) is displayed,
the user has specified full speed mode and
the VM is running in full speed mode.
<tr><td valign=top>
<image src="image/sp-ffmark1.png" style="padding:0.5ex; background-color:white">
<td valign=top>…
<td valign=top>
三角2つの場合、ユーザが高速モードを指定していて、等速モードで実行中です。
以下のいずれかの状態で起きます。
<q>When an icon (two triangles) is displayed,
the user has specified full speed mode and
the VM is running in synchronized mode.
This will happen when any of the following occurs:</q>
<ul>
<li>キー押下中(後述)あるいはキーボードブザー発声中
<q>When any keys are pressed (see below),
or the keyboard buzzer is sounding.
<li>STOP instruction on m68k.
<li>pseudo STOP state on m88k.
</ul>
</table>
</div>

<h5><?php sub(); ?> キー入力モード <q>Key input mode</q></h5>
<div class="main">
LUNA も X68030 もキーボードは通常日本語キーボードです
(LUNA には英語キーボードも存在するようですが…)。
そのため、ホストが日本語キーボードの場合とそうでない場合のために
2つの入力モードを用意しています。
どちらも一長一短があります。
<q>LUNA and X68030 has the Japanese keyboard
(though I ever heard there are US keyboard for LUNA...).
So nono provides two input modes
for Japanese keyboard users and non Japanese keyboard users.
Both have advantages and disadvantages.</q>
<p>
JP キーボードモードは、ホストが日本語キーボードであることを前提に
対応するキーの押下、開放をその都度 VM に入力します。
メリットとしては動作が実機に近くなり、
特にキーリピート(をハードウェアが行わないこと)も忠実に再現可能です。
デメリットは ALT + TAB やアクセラレータキーでメニューを開く場合などに
キー入力が残ることがあることです
(キーを押した後でフォーカスが外れるとキーを離したことが
アプリケーションに通知されないため)。
またホストが日本語キーボードでなければ使いづらいです。
<q>JP keyboard mode assumes that the host uses the Japanese keyboard.
Each time you presses and releases a key,
nono sends the corresponding key-pressed or key-released to the VM.
One advantage is that it's close to the real machine behavior,
and another is that it can reproduce
the LUNA keyboard itself does not perform a key repeat.
On the other hand,
one disadvantage is that key sometimes remains pressed,
especially when you enter ALT + TAB to switch the application
or enter accelerator key to open the menu
(If the application loses the focus after the key is pressed,
the application will not receive the subsequent key release).
And another disadvantage is that
it will be hard to use for non Japenese keyboard users.</q>
<p>
一方、キャラクタ入力モードは、ホストから入力された「文字」を
再び LUNA/X68030 のキーコードに解釈し直して VM に入力するモードです。
例えばゲスト OS に "@" を入力したい場合、
US キーボードなら [SHIFT] + [2] を、
JP キーボードなら [@] キーを押しますが、
どちらも VM には [@] キーの押下、[@] キーの開放が連続して送られます。
US キーボードで [SHIFT] を押したことは通知されません。
逆に "=" を入力したい場合、
US キーボードなら [=] キーを、
JP キーボードなら [SHIFT] + [-] を押しますが、
どちらも VM には
[SHIFT]押下 → [-]押下 → [-]開放 → [SHIFT]開放のシーケンスが送られます。
このようにホストキーボードのレイアウトによらず、
入力したい文字が入力できるようになり、
通常のタイピングで困ることはないと思います。
デメリットは、通常の文字入力以外のタイピングはほぼ出来ないことです。
[SHIFT] キーを押したままにすることや、
文字が入力できないキーの組み合わせは入力出来ません。
もう一つのデメリットはキーリピート(をハードウェアが行わないこと)が再現できず、
ホスト側のキーリピートが効いてしまうことです
(本来キーリピートが動作しない ROM やブートローダでもキーリピートが効くことは
一般ユーザからはメリットに感じられるかも知れませんが)。
<q>In contrast, in the character mode, when you type a character,
nono re-converts it into the LUNA/X68030's key code and sends it to the VM.
For example if you want to type "@" in the guest OS,
you would press [SHIFT] + [2] on the US keyboard or [@] on the JP keyboard.
In both cases, the sequence of
[@]-pressed and [@]-released will be sent to the VM.
Note that the fact that you pressed [SHIFT] key on the US keyboard
will not be sent to the VM.
Another example, if you want to type "=",
you would press [=] on the US keyboard or [SHIFT] + [-] on the JP keyboard.
In both cases, the sequence of
[SHIFT]-pressed, [-]-pressed, [-]-released, and [SHIFT]-released
will be sent to the VM.
Thus, you can type what character you want to type,
regardless of your keyboard layout.
Of course there are some disadvantages.
Firstly, it's hard to type any special characters,
for example, you can not type any key conbinations
that don't generate a character.
Or you can not hold [SHIFT] key pressed.
Secondly, the key repeat works (by the host),
even though it should not work as a correct emulation
(Although some of you may feel it an advantage).</q>
</div>

<h5><?php sub(); ?> キャラクタ入力モードのキーコード対応表
<q>Keycode table in the character mode</h5>
<div class="main">
キャラクタ入力モードでの、ホストから入力した文字と
VM に送信されるキーの対応は次の通りです。
ファンクションキーとカーソルキーは文字ではありませんが
特別に対応しています。
ここに記載のないゲストキーはソフトウェアキーボードからのみ入力できます。
<q>The following table shows the correspondence between
the character that typed from the host
and the key that will be sent to the VM.
nono can recognize the function keys and arrow keys.
The keys that not listed in this table can
only be typed from the software keyboard window.</q>
<p>
<center>
<style type="text/css">
.keytable-whole {
	display: inline-block;
	vertical-align: top;
	border: solid 1px;
}
.keytable-middle {
	margin-left: 1em;
	margin-right: 1em;
}

.keytable-head {
	text-align: left;
	border-bottom: solid 1px;
}
.keytable-headl {
	padding-right: 1em;
}

.keytable-tdl {
	padding-right: 1em;
}
.keytable-tdr {
}
</style>
<?php
	$str = <<<__EOM__
		<tt>a</tt> 〜 <tt>z</tt>	[A] 〜 [Z]
		<tt>0</tt> 〜 <tt>9</tt>	[0] 〜 [9]
		SPACE		[SPACE]
		<tt>-</tt>	[-]
		<tt>^</tt>	[^]
		<tt>&bsol;</tt>	[&bsol;]
		<tt>@</tt>	[@]
		<tt>[</tt>	[[]
		<tt>;</tt>	[;]
		<tt>:</tt>	[:]
		<tt>]</tt>	[]]
		<tt>,</tt>	[,]
		<tt>.</tt>	[.]
		<tt>/</tt>	[/]

		<tt>A</tt> 〜 <tt>Z</tt>	[SHIFT]+[A] 〜 [SHIFT]+[Z]
		<tt>!</tt>	[SHIFT]+[1]
		<tt>"</tt>	[SHIFT]+[2]
		<tt>#</tt>	[SHIFT]+[3]
		<tt>$</tt>	[SHIFT]+[4]
		<tt>%</tt>	[SHIFT]+[5]
		<tt>&amp;</tt>	[SHIFT]+[6]
		<tt>'</tt>	[SHIFT]+[7]
		<tt>(</tt>	[SHIFT]+[8]
		<tt>)</tt>	[SHIFT]+[9]
		<tt>=</tt>	[SHIFT]+[-]
		<tt>~</tt>	[SHIFT]+[^]
		<tt>|</tt>	[SHIFT]+[&bsol;]
		<tt>`</tt>	[SHIFT]+[@]
		<tt>{</tt>	[SHIFT]+[[]
		<tt>+</tt>	[SHIFT]+[;]
		<tt>*</tt>	[SHIFT]+[:]
		<tt>]</tt>	[SHIFT]+[]]
		<tt>&lt;</tt>	[SHIFT]+[,]
		<tt>&gt;</tt>	[SHIFT]+[.]
		<tt>?</tt>	[SHIFT]+[/]
		<tt>_</tt>	[SHIFT]+[_]

		F1 〜 F10	[PF1] 〜 [PF10]
		F11		[PF10]
		↑		[↑]
		←		[←]
		→		[→]
		↓		[↓]
		<tt>^@</tt>	[CTRL]+[@]
		<tt>^A</tt> 〜 <tt>^Z</tt>	[CTRL]+[A] 〜 [CTRL]+[Z]
		<tt>^[</tt>	[ESC]
		<tt>^\</tt>	[CTRL]+[&bsol;]
		<tt>^]</tt>	[CTRL]+[]]
		<tt>^^</tt>	[CTRL]+[^]
		<tt>^_</tt>	[CTRL]+[_]
		<tt>^H</tt>	[BS]
		<tt>^I</tt>	[TAB]
		<tt>^M</tt>	[Enter]
__EOM__;
	$table = array();
	$table[0] = array();
	$table[1] = array();
	$table[2] = array();
	$i = 0;
	foreach (split("\n", $str) as $s) {
		if ($s == "") {
			$i++;
			continue;
		}
		$s = preg_replace("/^\t+/", "", $s);
		list ($hkey, $gkey) = preg_split("/\t+/", $s);
		$table[$i][] = array($hkey, $gkey);
	}

	for ($j = 0; $j < count($table); $j++) {
		switch ($j) {
		 case 0:	$classname = "left";	break;
		 default:	$classname = "middle";	break;
		 case 3:	$classname = "right";	break;
		}
		print "<table class='keytable-whole keytable-${classname}'";
		print " cellspacing=0 cellpadding=1>\n";
		print "<tr>";
		print "<th class='keytable-head keytable-headl'>HostChar";
		print "<th class='keytable-head keytable-headr'>GuestKey\n";
		for ($i = 0; $i < count($table[$j]); $i++) {
			list ($h, $g) = $table[$j][$i];
			print "<tr>";
			print "<td class='keytable-tdl'>{$h}";
			print "<td class='keytable-tdr'>${g}\n";
		}
		print "</table>\n";
	}
?>
</center>
</div>

<h5><?php sub(); ?> JP キーボードモードのキーコード対応表
<q>Keycode table in the JP keyboard mode</q></h5>
<div class="main">
JP キーボードモードでの、ホストキーと VM に送信されるキーの対応は次の通りです。
ここに記載のないキーはソフトウェアキーボードからのみ入力出来ます。
<q>The following table shows the correspondence between
the host key and the guest key.
The keys that not listed in this table can only be enter from
the software keyboard window.</q>
<p>
<center>
<?php
	$str = <<<__EOM__
		[0] 〜 [9]		[0] 〜 [9]
		[A] 〜 [Z]		[A] 〜 [Z]
		[-]				[-]
		[^]				[^]
		[&bsol;]		[&bsol;]
		[@]				[@]
		[[]				[[]
		[;]				[;]
		[:]				[:]
		[]]				[]]
		[,]				[,]
		[.]				[.]
		[/]				[/]
		[_]				[_]

		[ESC]			[ESC]
		[TAB]			[TAB]
		[CTRL]			[CTRL]
		[BS]			[BS]
		[Enter]			[Enter]
		[F1] 〜 [F10]	[PF1] 〜 [PF10]
		[F11]			[PF10]
		[SHIFT-L]		[SHIFT-L]
		[SHIFT-R]		[SHIFT-R]
		[↑]			[↑]
		[←]			[←]
		[→]			[→]
		[↓]			[↓]

		[DEL]			Numpad[DEL]
		Numpad[0]〜[9]	Numpad[0]〜[9]
		Numpad[+]		Numpad[+]
		Numpad[-]		Numpad[-]
		Numpad[*]		Numpad[*]
		Numpad[/]		Numpad[/]
		Numpad[=]		Numpad[=]
		Numpad[.]		Numpad[.]
		Numpad[Enter]	Numpad[Enter]
__EOM__;
	$table = array();
	$table[0] = array();
	$table[1] = array();
	$table[2] = array();
	$i = 0;
	foreach (split("\n", $str) as $s) {
		if ($s == "") {
			$i++;
			continue;
		}
		$s = preg_replace("/^\t+/", "", $s);
		list ($hkey, $gkey) = preg_split("/\t+/", $s);
		$table[$i][] = array($hkey, $gkey);
	}

	for ($j = 0; $j < count($table); $j++) {
		switch ($j) {
		 case 0:	$classname = "left";	break;
		 case 1:	$classname = "middle";	break;
		 case 2:	$classname = "right";	break;
		}
		print "<table class='keytable-whole keytable-${classname}'";
		print " cellspacing=0 cellpadding=0>\n";
		print "<tr>";
		print "<th class='keytable-head keytable-headl'>HostKey";
		print "<th class='keytable-head keytable-headr'>GuestKey\n";
		for ($i = 0; $i < count($table[$j]); $i++) {
			list ($h, $g) = $table[$j][$i];
			print "<tr>";
			print "<td class='keytable-tdl'>{$h}";
			print "<td class='keytable-tdr'>${g}\n";
		}
		print "</table>\n";
	}
?>
</center>
</div>

<h5><?php sub(); ?> キーリピート <q>Key Repeat</q></h5>
<div class="main">
LUNA では、キーボードがハードウェア側でキーリピートを行わず、
ソフトウェア(OS など)がキーリピートの処理を行います。
そのため、キーリピートを実装していない LUNA の PROM や
NetBSD/luna68k のブートローダではキーリピートは起きず、
キーリピートを実装している NetBSD/luna68k カーネルではキーリピートが起こせます。
キーリピートを起こす間隔をゲストのソフトウェアが測定しているということは、
VM が高速動作するとキーリピートもそれに合わせて発生することになり、
これをホスト側から防ぐ手段はありません。
そこで nono ではキー入力が発生している間
(キーが一つでも押されてからキーが全て離されるまでの間)、
高速モードが指示されていても VM を一時的に等速モードに落として実行します。
上記のアイコンがそれを区別しているのはこのためです。
そのため、何らかの理由でキーが入りっぱなしになった場合
(ALT + TAB やアクセラレータキーでメニューを開くと起きがちです)
高速モードが抑制されたままということが起きえます。
その場合はソフトウェアキーボードからキー入力を解除するなどしてください。
<q>On LUNA,
key repeat is done by software(OS), not by the keyboard hardware.
For this reason,
key repeat doesn't occur on LUNA's PROM or NetBSD/luna68k's bootloader
that don't implement it, and
key repeat occurs on NetBSD/luna68k kernel that implements it.
Since the timing of key repeat is measured by the guest software,
if the VM is running faster than the real,
the key repeat will occur faster, too.
The host application doesn't have the way to avoid it.
Therefore, nono will temporarily suppress the full speed mode
while any keys are pressed.
That is why the above-mentioned icon distinguishes them.
If keys continue to be pressed for some reasons,
the VM also continues to run syncronized mode.
In this case, you can resolve it by using the software keyboard window.</q>
</div>



<a name="tryit"></a>
<h4>5. 実行してみる <q>Try it</q></h4>
<h5>5.1. NetBSD/luna68k を実行してみる <q>Try NetBSD/luna68k</q></h5>
<div class="main">
つついさんが NetBSD/luna68k 9.2 の liveimage を用意されています。
<q>Tsutsui-san has provided a liveimage of NetBSD/luna68k 9.2.</q>
<dl>
<dd><a href="https://twitter.com/tsutsuii/status/1436381589211017217"
>https://twitter.com/tsutsuii/status/1436381589211017217</a><br>
</dl>
ここではこれを起動してみます。<q>Let's try it.</q>
<br>
<ol>
<li>どこかに nono 用のディレクトリを用意し(例えば ~/nono/)、
その中に VM ディレクトリを用意します (例えば ~/nono/luna/)。
<q>Create a directory somewhere for nono (for example ~/nono/),
and create subdirectories for individual VMs in it
(for example ~/nono/luna/).</q>

<li>以下のリンクからイメージファイルをダウンロードして展開し、
VM ディレクトリ ~/nono/luna/ に置きます。
<q>Download imagefile from the following link,
extract it and place it in the VM directory, ~/nono/luna/.</q>
<blockquote>
<a href="http://teokurebsd.org/netbsd/liveimage/20210910-luna68k/"
>http://teokurebsd.org/netbsd/liveimage/20210910-luna68k/</a>
</blockquote>

<li>以下の内容の設定ファイル nono.cfg を VM ディレクトリ ~/nono/luna/
に作成します。
ここでは説明を簡単にするためネットワークなしにしていますが、
ネットワーク設定は必要に応じて行ってください。
<q>Create a configuration file nono.cfg in the VM directory, ~/nono/luna/,
with following contents.
By the way, to simplify the explanation, we assume there is no network here.
However, please configure the network if necessary.</q>
<blockquote class="file"><pre>
vmtype = luna
spc0-id6-image = hd,liveimage-luna68k-with-packages-20210910.img
hostnet-driver = none
</pre></blockquote>

<li><tt>nono -c ~/nono/luna</tt> で起動します
(VM ディレクトリに自動的に NVRAM.DAT が作られます)。
<q>Run as <tt>nono -c ~/nono/luna</tt>.
(NVRAM.DAT will be created automatically in the VM directory)</q>

<li>Emulated ROM Monitor が起動するので、
初回は以下のように入力すると NetBSD が起動します。
<q>The emulated ROM Monitor will be executed.
Then, only for the first time,
entering the following can boot NetBSD.</q>
<blockquote><pre>
k&#x23ce;
&#x23ce;
&#x23ce;
d&#x23ce;
boot&#x23ce;
g&#x23ce;
x&#x23ce;
</pre></blockquote>
画面はこんな感じのはずです (太字が入力部分)。
<q>You will see a screen like this.  The bold text indicates
the characters you need to enter.</q>
<blockquote class="file"><pre>
NONO <?=$nono_ver?> Emulated ROM Monitor for LUNA-I

** NVRAM Initialized.

&gt;<b>k&#x23ce;</b>
controller: dk  ?<b>&#x23ce;</b>
drive unit: 0  ?<b>&#x23ce;</b>
partition : c  ?<b>d&#x23ce;</b>
filename  : vmunix  ?<b>boot&#x23ce;</b>
&gt;<b>g&#x23ce;</b>
Loaded.  Entry point = $00700000
&gt;<b>x&#x23ce;</b>
</pre></blockquote>
この内容は NVRAM.DAT に記録されているので次回以降は直接 NetBSD が起動します。
<q>The information you have just entered is recorded in the NVRAM,
so next time it will boot NetBSD automatically.</q>
<li>初回起動時、
Updating fontconfig cache はあほみたいに時間がかかりますが、
nono がハングアップしてるわけではありません(>_<)。
また初回は起動後数時間めちゃくちゃ重たいですが、
これはバックグラウンドで makemandb が動くためで
nono のせいではありません(>_<)。
<q>At the first boot, you will see the console stops after printing
"Updating fontconfig cache".
This is because the infamous fontconfig takes very looooong time.
nono would not have hang-up.
In addition, at the first boot, you will feel it's too heavy for several hours.
This is because the infamous makemandb(8) runs heavily in the background
for a looooong time.
It's very sad to me that these two accidents which are far from the ideal
are the first experiences of newcomers.</q>

<li>終了する時は root
ユーザで &ldquo;<tt>shutdown -p now</tt>&rdquo; を実行してください。
LUNA はソフトウェアから電源オフでき、
VM の電源オフで nono も終了します。
<q>To quit, type &ldquo;<tt>shutdown -p now</tt>&rdquo; as the root user.
LUNA can be powered off by software,
and nono will terminate when the VM is powered off.</q>
</ol>
</div>

<h5>5.2. OpenBSD/luna88k を実行してみる <q>Try OpenBSD/luna88k</q></h5>
<div class="main">
あおやまさんが OpenBSD/luna88k 7.0 の liveimage を用意されています。
<q>Aoyama-san has provided a liveimage of OpenBSD/luna88k 7.0.</q>
<dl>
<dd><a href="https://twitter.com/ao_kenji/status/1473611893386661888"
>https://twitter.com/ao_kenji/status/1473611893386661888</a>
</dl>
ここではこれを起動してみます。<q>Let's try it.</q>
<br>
<ol>
<li>どこかに nono 用のディレクトリを用意し(例えば ~/nono/)、
その中に VM ディレクトリを用意します (例えば ~/nono/luna88k/)。
<q>Create a directory somewhere for nono (for example ~/nono/),
and create subdirectories for individual VMs in it
(for example ~/nono/luna88k/).</q>

<li>以下のリンクから liveimage-luna88k-raw-20211221.img.gz
をダウンロードして展開し、
VM ディレクトリ ~/nono/luna88k/ に置きます。
<q>Download liveimage-luna88k-raw-20211221.img.gz from the following link,
extract it and place it in the VM directory, ~/nono/luna88k/.</q>
<blockquote>
<a href="http://www.nk-home.net/~aoyama/liveimage/"
>http://www.nk-home.net/~aoyama/liveimage/</a>
</blockquote>

<li>以下の内容の設定ファイル nono.cfg を VM ディレクトリ ~/nono/luna88k/
に作成します。
ここでは説明を簡単にするためネットワークなしにしていますが、
ネットワーク設定は必要に応じて行ってください。
<q>Create a configuration file nono.cfg in the VM directory, ~/nono/luna88k/,
with following contents.
By the way, to simplify the explanation, we assume there is no network here.
However, please configure the network if necessary.</q>
<blockquote class="file"><pre>
vmtype = luna88k
spc0-id6-image = hd,liveimage-luna88k-raw-20211221.img
hostnet-driver = none
</pre></blockquote>

<li><tt>nono -c ~/nono/luna88k</tt> で起動します
(VM ディレクトリに自動的に NVRAM.DAT が作られます)。
<q>Run as <tt>nono -c ~/nono/luna88k</tt>.
(NVRAM.DAT will be created automatically in the VM directory)</q>
<li>Emulated ROM Monitor が起動するので、
初回は以下のように入力すると OpenBSD が起動します。
<q>The emulated ROM Monitor will be executed.
Then, only for the first time,
entering the following can boot OpenBSD.</q>
<blockquote><pre>
nvram boot_filename boot&#x23ce;
y
b&#x23ce;
</pre></blockquote>
画面はこんな感じのはずです (太字が入力部分)。
<q>You will see a screen like this.  The bold text indicates
the characters you need to enter.</q>
<blockquote class="file"><pre>
NONO <?=$nono_ver?> Emulated ROM Monitor for LUNA88K

** NVRAM Initialized.

N&gt;<b>nvram boot_filename boot&#x23ce;</b>
Update boot_filename : "vmunix" -&gt; "boot" (Y/[N]):<b>y</b>
Updated
N&gt;<b>b&#x23ce;</b>
</pre></blockquote>
この内容は NVRAM.DAT に記録されているので次回以降は直接 OpenBSD が起動します。
<q>The information you have just entered is recorded in the NVRAM,
so next time it will boot OpenBSD automatically.</q>

<li>終了する時は root
ユーザで &ldquo;<tt>shutdown -p now</tt>&rdquo; を実行してください。
LUNA-88K はソフトウェアから電源オフでき、
VM の電源オフで nono も終了します。
<q>To quit, type &ldquo;<tt>shutdown -p now</tt>&rdquo; as the root user.
LUNA-88K can be powered off by software,
and nono will terminate when the VM is powered off.</q>
</ol>
</div>

<h5>5.3 NetBSD/x68k を実行してみる <q>Try NetBSD/x68k</q></h5>
<div class="main">
つついさんが NetBSD/x68k 9.2 の liveimage を用意されています。
ここではこれを起動してみます。
<q>Tsutsui-san has provided a liveimage of NetBSD/x68k 9.2.  Let's try it.</q>
<br>
<ol>
<li>どこかに nono 用のディレクトリを用意し(例えば ~/nono/)、
その中に VM ディレクトリを用意します (例えば ~/nono/x68k/)。
<q>Create a directory somewhere for nono (for example ~/nono/),
and create subdirectories for individual VMs in it
(for example ~/nono/x68k/).</q>

<li>以下のリンクから
plain-install か with-packages のお好みのほうをダウンロードして展開し、
VM ディレクトリ ~/nono/x68k/ に置きます。
<q>Download your favorite one of plain-install or with-packages
from the following link,
extract it and place it in the VM directory, ~/nono/x68k/.</q>
<blockquote>
<a href="http://teokurebsd.org/netbsd/liveimage/20210613-x68k/"
>http://teokurebsd.org/netbsd/liveimage/20210613-x68k/</a>
</blockquote>

<li>以下の内容の設定ファイル nono.cfg を VM ディレクトリ ~/nono/x68k/
に作成します。
初回起動時は liveimage は接続しません。
<q>Create a configuration file nono.cfg in the VM directory, ~/nono/x68k/,
with following contents.
You don't need to add the liveimage first time.</q>
<blockquote class="file"><pre>
vmtype = x68030
</pre></blockquote>

<li><tt>nono -c ~/nono/x68k</tt> で起動します。
しばらくすると起動メディアが見付からず白帯が出ますので
メニューの「File &gt; Exit」か×ボタンで終了します。
これで IPLROM により初期化された SRAM が
VM ディレクトリに SRAM.DAT として保存されます。
<q>Run as <tt>nono -c ~/nono/x68k</tt>.
After a few seconds, a message will be displayed like this captured screen.
It says in Japanese that "Cannot boot from disk.  Please set correct one".
Then, terminate the nono by "File &gt; Exit" in menu or close button.
By this operation, SRAM will be initialized by X68030 IPLROM and
be saved as SRAM.DAT in the VM directory.</q>
<center><img src="image/x68030-nodisk.png"></center>

<li>ここで設定ファイル nono.cfg に先程ダウンロードしてきたディスクイメージを追加します (2行目)。
<q>Then, add a disk image downloaded earlier to configuration file nono.cfg
(see 2nd line).</q>
<blockquote class="file"><pre>
vmtype = x68030
spc0-id0-image = hd,liveimage-x68k-with-packages-20210613.hds
</pre></blockquote>

<li>もう一度 <tt>nono -c ~/nono/x68k</tt> で起動すると、
HDD から NetBSD/x68k が起動します。
次回以降はこれだけで直接 NetBSD が起動します。
<q>Again, run as <tt>nono -c ~/nono/x68k</tt>.
NetBSD/x68k will be booted from HDD.
Next time, it will boot NetBSD automatically.</q>

<li>初回起動後数時間はめちゃくちゃ重たいですが、
これはバックグラウンドで makemandb が動くためで
nono のせいではありません(>_<)。
<q>At the first boot, you will feel it's too heavy for several hours.
This is because the infamous makemandb(8) runs heavily in the background
for a looooong time.
It's very sad to me that an accident which is far from the ideal
is the first experiences of newcomers.</q>

<li>NetBSD が起動した後なら (正確には powerd(8) が起動していれば)、
終了する時はメニューの「デバイス &gt; 操作 &gt; 電源ボタンを押す」
でシャットダウンして電源オフになり、
VM の電源オフで nono も終了します。
<q>After NetBSD boots up (more precisely, if powerd(8) is running),
you can shutdown and power off by
"Device &gt; Operation &gt; Push Power button" on menu,
and nono will terminate when the VM is powered off.</q>
</ul>
</div>


<a name="network"></a>
<h4>6. ホストネットワーク設定例 <q>Example of host network setup</q></h4>
<div class=main>
wm0 を持つ NetBSD ホストに tap(4) デバイスを用いて
nono のゲスト OS を接続する場合の設定例です。
<ol>
<li>
設定ファイル nono.cfg に以下の行を追加します
(と言いつつ NetBSD では書かなくてもデフォルトでこの動作になりますが)
<q>Add the following line to configuration file, nono.cfg.
(Although you don't need to write it since these are default behavior
on NetBSD)</q>
<blockquote class="file"><pre>
hostnet-driver = tap
hostnet-tap-devpath = auto
</pre></blockquote>

<li>デフォルトでは /dev/tap は一般ユーザからアクセスできないので、
chmod で適当にパーミッションを与えます。
番号の付いていないほうの /dev/tap だけでいいです。
sysinst 等で OS をアップグレードするとパーミッションが 600
に戻るのがハマりポイントです。
<q>By default, /dev/tap is only accessible to privileged user.
You need to chmod /dev/tap (without unit number) appropriately.
Note that upgrading using sysinst always reset the permission to 600.</q>

<li>bridge(4) インタフェースを作成し、
ホストの外部(物理)インタフェースをブリッジに追加しておきます。
<q>Create a bridge(4) interface, and
add your physical interface to the bridge.</q>
<blockquote class="cons"><pre>
# ifconfig bridge0 create
# brconfig bridge0 add wm0
</pre></blockquote>
常用するなら /etc の設定ファイルに書いておきましょう。
<q>If you use it regularly,
you can put configuration file into /etc.</q>
<blockquote>
/etc/ifconfig.bridge0
<pre class=file>
create
up
!/sbin/brconfig $int add wm0
</pre>
<pre class=cons>
# /etc/rc.d/network restart
</pre>
</blockquote>

<li>
一般ユーザに戻って、
VM ディレクトリかその親ディレクトリに
次のような 2つのスクリプトを用意します。
nono は tap(4) をオープンし、
そのデバイス名を引数にこれらのスクリプトを呼びます。
sudo の設定は別途行ってください。
<q>Return to non-privileged user, and
create following two scripts in the VM directory or its parent directory.
nono will open tap(4) and
invoke these scripts with the name of the device as an argument.
In addition, you need to set up sudo separately.</q>
<blockquote>
nono-ifup
<pre class=file>
#!/bin/sh
sudo /sbin/ifconfig $1 up
sudo /sbin/brconfig bridge0 add $1
</pre></blockquote>

<blockquote>
nono-ifdown
<pre class=file>
#!/bin/sh
sudo /sbin/brconfig bridge0 delete $1
sudo /sbin/ifconfig $1 down
</pre></blockquote>

<blockquote class="cons"><pre>
% chmod +x nono-ifup nono-ifdown
</pre></blockquote>

<li>nono を起動し、
メニューの「モニタ &gt; ホスト &gt; ホストネットワーク」を開いて
HostNet Driver: tap になっていれば動いてるはずです。
<q>Run nono,
and open "Monitor &gt; Host &gt; Host Network" window from menu.
It's OK if you can see "HostNet Driver: tap".</q>

</ol>

</div>

<a name="knownissues"></a>
<h4>7. 既知の問題 <q>Known Issues</q></h4>
<div class="main">
<ul>
<li>デバッガのコンソール入力で、
バックスペースやカーソルキーの入力に対する表示が動作していません。
<q>In debugger console, some special characters like backspace or cursor keys
aren't printed correctly.</q>
(since ver 0.3.0)
<li>ホストが VirtualBox (on Windows?) で、
VirtualBox の「マウス統合」が有効にしている場合、
nono の「マウスモード」でマウスポインタが正しく移動しません。
<q>If the host OS is running on VirtualBox (on Windows?) and
"Mouse integration" on VirtualBox is enabled,
the mouse pointer will not move as expected in nono's Mouse Mode.</q>
(2021/12/30)

<li>Wayland の場合にウィンドウサイズが正しく設定されない場合があります。
wxWidgets (3.0.5) のバグと思われます。
<q>On Wayland, window size may be incorrect.
This may be due to wxWidgets (3.0.5) bug.</q>
(2021/12/30)

</ul>
</div>

<a name="migrate"></a>
<h4>8. 過去のバージョンからの移行方法 <q>How to migrate from old versions</q></h4>
<div class="main">
バージョンアップに伴い設定ファイル等に非互換が発生する場合があります。
その場合は以下の移行方法を参照して設定ファイル等を更新してください。
<q>Some versions may have incompatibilities in the configuration files, etc.
In such case, you may need to upgrade it by referring the following link.</q>
<ul>
<li><a href="upgrade-0.3.0.html">From ver 0.2.x to ver 0.3.0</a>
<li><a href="upgrade-0.2.0.html">From ver 0.1.x to ver 0.2.0</a>
</ul>
</div>

<a name="changes"></a>
<h4>9. 変更履歴 <q>Changes</q></h4>
<div class="main">
See <a href="changes.html">changes.html</a>.
</div>

<a name="license"></a>
<h4>10. 連絡先、ライセンス等 <q>Contact, License, etc</q></h4>

<h5>10.1. ライセンス <q>License</q></h5>
<div class="main">
See <a href="nono-license.txt">nono-license.txt</a>.
</div>
<br>
<div class="main">
また nono は無償公開されている X68030 の IPLROM を使用しています。
<a href="fsharp-license.txt">許諾条件</a>を読んでください。
<q>nono uses X68030 IPLROM image distributed without charge.
You must read
<a href="fsharp-license.txt">the license agreement</a>
(even though it's Japanese original text only).</q>
</div>

<h5>10.2. 連絡先 <q>Contact us</q></h5>
<div class="main">
バグ報告などは以下にお願いします。日本語でおk。
<q>If you find any problems, please let me know.
You may write in English.</q><br>
<a href="https://github.com/isaki68k/nono-issue/issues"
>https://github.com/isaki68k/nono-issue/issues</a>
</div>

<h5>10.3. パッチの提供について <q>About contributes</q></h5>
<div class="main">
パッチを提供してくださる場合は以下に同意したものとします。
<q>If you provide a patch to nono, we assume that you agree the following
conditions:</q>
<ul>
<li>成果物が nono のライセンスに従って運用あるいは配布されること。
<q>All your work are operated or distributed under the nono license.</q>
<li>ライセンスが将来変わる可能性があること。
<q>The license may be changed in the future.</q>
<li>著作部分に関して著作者人格権を行使しないこと。
<q>Do not exercise your author's rights.</q>
</ul>
<ul>
</div>

<h5>10.4. Acknowledgements</h5>
<div class="main">
nono は以下の広告条項を含むソースコードを利用しています。
<q>nono uses source code with the following advertising clause.</q>
<blockquote>
This product includes software developed by Gordon Ross<br>
This product includes software developed by the University of California, Lawrence Berkeley Laboratory.<br>
</blockquote>
</div>


<hr>
nono project
</body>
</html>
