//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// エリアセット
//

#include "areaset.h"
#include "mpu.h"

// グローバル参照用
AreaSetDevice *gAreaSet;

// コンストラクタ
AreaSetDevice::AreaSetDevice()
	: inherited("AreaSet")
{
	devaddr = baseaddr;
	devlen  = 0x2000;
}

// デストラクタ
AreaSetDevice::~AreaSetDevice()
{
	gAreaSet = NULL;
}

// リセット
void
AreaSetDevice::ResetHard(bool poweron)
{
	limit = 0;
}

uint64
AreaSetDevice::Read8(uint32 addr)
{
	switch (addr) {
	 case 0xe86001:
		return limit;
	}
	return (uint64)-1;
}

uint64
AreaSetDevice::Read16(uint32 addr)
{
	return 0xff00 | Read8(addr + 1);
}

uint64
AreaSetDevice::Write8(uint32 addr, uint32 data)
{
	gMPU->AddCycle(7);	// InsideOut p.135

	switch (addr) {
	 case 0xe86001:
		putlog(0, "Write $%02x (NOT IMPLEMENTED)", data);
		limit = data;
		return 0;
	 default:
		break;
	}
	return (uint64)-1;
}

uint64
AreaSetDevice::Write16(uint32 addr, uint32 data)
{
	return Write8(addr + 1, data);
}

uint64
AreaSetDevice::Peek8(uint32 addr)
{
	switch (addr) {
	 case 0xe86001:
		return limit;
	 default:
		return 0xff;
	}
}
