//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// Ethernet 基本クラス
//

#include "ethernet.h"
#include "hostnet.h"
#include "scheduler.h"

// グローバル参照用
EthernetDevice *gEthernet;

// コンストラクタ
EthernetDevice::EthernetDevice(const std::string& objname_)
	: inherited(objname_)
{
}

// デストラクタ
EthernetDevice::~EthernetDevice()
{
	if ((bool)hostnet) {
		hostnet->SetCallbackDevice(NULL);
	}
	gEthernet = NULL;
}

// 動的なコンストラクション
bool
EthernetDevice::Create()
{
	// ホストドライバを作成してこのデバイスと紐付ける
	hostnet.reset(new HostNetDevice());

	hostnet->SetRxCallback(ToDeviceCallback(&EthernetDevice::HostRxCallback));
	hostnet->SetCallbackDevice(this);

	return true;
}

// これは Host スレッドから呼ばれる
void
EthernetDevice::HostRxCallback()
{
	// スレッドを超えるためにメッセージを投げる
	gScheduler->SendMessage(MessageID::HOSTNET_RX);
}
