//
// nono
// Copyright (C) 2022 nono project
// Licensed under nono-license.txt
//

//
// メッセージ
//

#pragma once

#include "device.h"

class MessageID
{
 public:
	enum {
		NONE = 0,

		RESET,
		POWEROFF_EXIT,
		POWEROFF_RESTART,
		POWER_BUTTON,

		SCSIDEV_LOAD_0,
		SCSIDEV_LOAD_1,
		SCSIDEV_LOAD_2,
		SCSIDEV_LOAD_3,
		SCSIDEV_LOAD_4,
		SCSIDEV_LOAD_5,
		SCSIDEV_LOAD_6,
		SCSIDEV_LOAD_7,
		SCSIDEV_UNLOAD_0,
		SCSIDEV_UNLOAD_1,
		SCSIDEV_UNLOAD_2,
		SCSIDEV_UNLOAD_3,
		SCSIDEV_UNLOAD_4,
		SCSIDEV_UNLOAD_5,
		SCSIDEV_UNLOAD_6,
		SCSIDEV_UNLOAD_7,

		FDD_LOAD_0,
		FDD_LOAD_1,
		FDD_LOAD_2,
		FDD_LOAD_3,
		FDD_UNLOAD_0,
		FDD_UNLOAD_1,
		FDD_UNLOAD_2,
		FDD_UNLOAD_3,

		HOSTCOM_RX,
		HOSTNET_RX,

		KEY_INPUT,

		NMI,

		SCHEDULER_SYNCMODE,

		MPU_TRACE,

		MAX,
	};

 public:
	MessageID() {
		id = NONE;
	}
	MessageID(int id_) {
		id = id_;
	}

	MessageID& operator=(int id_) {
		id = id_;
		return *this;
	}
	operator int() const {
		return id;
	}

	static MessageID SCSIDEV_LOAD(int n) {
		return MessageID(SCSIDEV_LOAD_0 + n);
	}
	static MessageID SCSIDEV_UNLOAD(int n) {
		return MessageID(SCSIDEV_UNLOAD_0 + n);
	}

	static MessageID FDD_LOAD(int n) {
		return MessageID(FDD_LOAD_0 + n);
	}
	static MessageID FDD_UNLOAD(int n) {
		return MessageID(FDD_UNLOAD_0 + n);
	}

 private:
	int id;
};

using MessageCallback_t = void (Device::*)(MessageID, uint32);
#define ToMessageCallback(f) static_cast<MessageCallback_t>(f)
