//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// MPU (M88xx0)
//

#pragma once

#include "event.h"
#include "mpu.h"
#include "m88100.h"

class MPU88xx0Device : public MPUDevice
{
	using inherited = MPUDevice;
 public:
	MPU88xx0Device();
	virtual ~MPU88xx0Device() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	// 現在実行中の命令の PC を取得
	uint32 GetPPC() const override { return cpu->GetXIP(); }

	// 現在の VBR を取得
	uint32 GetVBR() const override { return cpu->GetVBR(); }

	// 現在アクセス中の論理アドレス、物理アドレスを取得。
	uint32 GetLaddr() const override;
	uint32 GetPaddr() const override;

	// アクセスウェイトを加算
	void AddCycle(int32 cycle) override;

	// 割り込み発生を MPU に通知
	void Interrupt(int level) override;

	// DOS call エミュレーションのコールバックを指定する
	void SetFLineCallback(bool (*callback)(m88kcpu *, void *), void *arg);

	// CPU コアを取得 (主にデバッグ用途)
	m88kcpu *GetCPU() const { return cpu.get(); }

	// MPU からのアクセスをエミュレートする。
	// アドレスは論理アドレス。
	// 空間は現在のデータ空間。
	// ミスアラインドアクセスも可能。
	// アクセスエラーは -1 を返す。
	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;

	// コアからスケジューラへの動作モード変更通知
	void NotifyCPUMode(uint32 new_mode);

 private:
	// リセット例外コールバック
	void ResetCallback(Event& ev);

	// 命令実行系コールバック
	void ExecNormal(Event& ev);
	void ExecStop(Event& ev);
	void ExecTrace(Event& ev);

	// トレース設定。デバッガから呼び出される。
	void SetTrace(bool trace_on) override;

	// レジスタモニター
	DECLARE_MONITOR_CALLBACK(MonitorUpdate);
	Monitor monitor { this };

	// イベントコールバック
	EventCallback_t exec_normal {};
	EventCallback_t exec_stop {};

	std::unique_ptr<m88kcpu> cpu {};
};

#define gMPU88xx0	(dynamic_cast<MPU88xx0Device *>(gMPU))
