//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// メインメモリ
//

#pragma once

#include "device.h"

class LoadInfo;

class RAMDevice : public IODevice
{
	using inherited = IODevice;
 public:
	RAMDevice();
	virtual ~RAMDevice() override;

	bool Init() override;
	void ResetHard(bool poweron) override;

	uint64 Read8(uint32 addr) override;
	uint64 Read16(uint32 addr) override;
	uint64 Read32(uint32 addr) override;
	uint64 Write8(uint32 addr, uint32 data) override;
	uint64 Write16(uint32 addr, uint32 data) override;
	uint64 Write32(uint32 addr, uint32 data) override;
	uint64 Peek8(uint32 addr) override;

	// RAM 容量(バイト単位)を取得
	int GetSize() const	{ return ram_size; }
	// RAM 容量(MB単位)を取得
	int GetSizeMB() const { return (uint)ram_size / 1024 / 1024; }

	// アクセスウェイトを設定
	void SetWait(uint32 wait);

	// path で指定されたホストファイルをロードする
	bool LoadFromFile(LoadInfo *);
	// data, size で指定されたバッファをロードする
	bool LoadFromData(LoadInfo *);

 private:
	bool LoadGzFile(LoadInfo *);
	bool Load_aout(LoadInfo *);
	bool Load_elf32(LoadInfo *);
	bool Load_elf32_exec(LoadInfo *);
	bool Load_elf32_rel(LoadInfo *);

	// RAM 容量(バイト単位)。ram[] の確保したバイト数
	int ram_size {};

	uint32 read_wait {};
	uint32 write_wait {};
};

// 実行ファイルを読み込む際の受け渡しパラメータ
class LoadInfo {
 public:
	LoadInfo();
	LoadInfo(const char *path_);
	LoadInfo(const char *path_, const uint8 *data_, size_t size_);

	// ファイルから読み込む時はファイルパスを指定する。
	// メモリから読み込む時はエラーの時に表示される名前を指定する。
	const char *path {};

	// データ本体とそのサイズ。メモリから読み込む時には指定する。
	// ファイルから読み込む時は内部で使用。
	const uint8 *data {};
	size_t size {};

	// 読み込んだファイルのエントリポイント。
	uint32 entry {};

	// 実際にメモリにロードしたサイズ。
	// たぶん ELF 実行ファイルを読み込んだ時のみ有効?
	uint32 esym {};
};

extern std::unique_ptr<uint8[]> mainram;
extern RAMDevice *gRAM;
