//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// ビデオコントローラ
//

#pragma once

#include "device.h"
#include "color.h"

class VideoCtlrDevice : public IODevice
{
	using inherited = IODevice;

	static const int baseaddr = 0xe82000;

	struct videoctlr {
		uint16 r0;
		uint16 r1;
		uint16 r2;
	};

 public:
	VideoCtlrDevice();
	virtual ~VideoCtlrDevice() override;

	void ResetHard(bool poweron) override;

	// 加工済みテキストパレットの先頭アドレスを取得
	// レンダラとかからの参照用
	const ColorXBGR *GetHostPalette() const { return cooked_palette; }

 protected:
	// BusIO インタフェース
	static const uint NPORT = 4096;
	uint64 Read(uint32 offset);
	uint64 Write(uint32 offset, uint32 data);
	uint64 Peek(uint32 offset);

 private:
	void SetR0(uint32 data);
	void SetR1(uint32 data);
	void SetR2(uint32 data);
	void MakePalette();

	struct videoctlr videoctlr {};
	union {
		uint8 b[1024];
		uint16 w[512];
	} palette {};

	// ホスト形式のテキストパレット (レンダラ用に加工したもの)
	ColorXBGR cooked_palette[16] {};
};

extern VideoCtlrDevice *gVideoCtlr;
