//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// VM (基本クラス)
//

#pragma once

#include "device.h"

// ヘッダの依存関係を減らすための細工。
// o pNAME はメンバ変数としてヘッダで宣言してあるが、vm*.h で各デバイスヘッダ
//   を include したくないので、派生型ではなく Device* の unique_ptr にしたい。
// o gNAME は各デバイスヘッダが持つ派生型のグローバルポインタ。
//   ベース型ではなく派生型で参照したい。また使いたい人が使いたいヘッダだけを
//   include すれば済むよう、vm*.h とは切り離したい。
#define NEWDV(NAME, NEW_CTOR)	do {	\
	__CONCAT(p,NAME).reset(NEW_CTOR);	\
	__CONCAT(g,NAME) = \
		(decltype(__CONCAT(g,NAME)))(__CONCAT(p,NAME).get());	\
} while (0)

class VM
{
 public:
	VM();
	virtual ~VM();

	// 明示的な後始末
	void Dispose();

	// 動的なコンストラクション
	bool Create();

	// 初期化
	virtual bool Init();

	// スレッド開始
	bool StartThread();

	// 設定の適用
	bool Apply();

	// ブートページを ROM に切り替える
	void SwitchBootPageToROM(Device *parent) { SwitchBootPage(parent, true); }
	// ブートページを RAM に切り替える
	void SwitchBootPageToRAM(Device *parent) { SwitchBootPage(parent, false); }

	// MPU の RESET 命令によるリセット (機種ごとに異なるため)
	virtual void ResetByMPU();

 protected:
	// ブートページを切り替える (内部用)
	virtual void SwitchBootPage(Device *parent, bool isrom);

	std::unique_ptr<Device> pBusErr {};
	std::unique_ptr<Device> pDebugger {};
	std::unique_ptr<Device> pInterrupt {};
	std::unique_ptr<Device> pKeyboard {};
	std::unique_ptr<Device> pMPU {};
	std::unique_ptr<Device> pNMI {};
	std::unique_ptr<Device> pPower {};
	std::unique_ptr<Device> pRenderer {};
	std::unique_ptr<Device> pRAM {};
	std::unique_ptr<Device> pScheduler {};
	std::unique_ptr<Device> pSignalThread {};
	std::unique_ptr<Device> pSync {};
};

extern VM *gVM;
