//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// VM (LUNA)
//

#pragma once

#include "vm.h"

class VM_LUNA : public VM
{
	using inherited = VM;
 public:
	VM_LUNA();
	virtual ~VM_LUNA() override;

	bool Init() override;

 protected:
	// ブートページを切り替える。
	void SwitchBootPage(Device *parent, bool isrom) override;

	// デバイスマップから devtable を作成
	void MakeDevtable(const uint8 *maptable);

	// ブートページ切り替え用 (unique_ptr ではない)
	IODevice *ram0 {};

	std::unique_ptr<Device> pBT454 {};
	std::unique_ptr<Device> pCRTC2 {};
	std::unique_ptr<Device> pEthernet {};
	std::unique_ptr<Device> pLCD {};
	std::unique_ptr<Device> pNopIO {};
	std::unique_ptr<Device> pPartialRAM {};
	std::unique_ptr<Device> pPlaneVRAM {};
	std::unique_ptr<Device> pPROM {};
	std::unique_ptr<Device> pPROM0 {};
	std::unique_ptr<Device> pPIO0 {};
	std::unique_ptr<Device> pPIO1 {};
	std::unique_ptr<Device> pRTC {};
	std::unique_ptr<Device> pSIO {};
	std::unique_ptr<Device> pSPC {};
	std::unique_ptr<Device> pSysClk {};
	std::unique_ptr<Device> pTAS {};
	std::unique_ptr<Device> pZRAM {};
};

class VM_LUNA1 final : public VM_LUNA
{
	using inherited = VM_LUNA;
 public:
	VM_LUNA1();
	~VM_LUNA1() override;

 private:
	static const uint8 maptable[256];
};

class VM_LUNA88K final : public VM_LUNA
{
	using inherited = VM_LUNA;
 public:
	VM_LUNA88K();
	~VM_LUNA88K() override;

 private:
	std::unique_ptr<Device> pCMMU {};
	std::unique_ptr<Device> pFuseROM {};

	static const uint8 maptable[256];
};
