//
// nono
// Copyright (C) 2021 nono project
// Licensed under nono-license.txt
//

//
// バックバッファが BitmapRGB なパネル
//

#include "wxbitmappanel.h"
#include "wxcolor.h"

// イベントテーブル
wxBEGIN_EVENT_TABLE(WXBitmapPanel, inherited)
	EVT_SIZE(WXBitmapPanel::OnSize)
	EVT_PAINT(WXBitmapPanel::OnPaint)
wxEND_EVENT_TABLE()

// コンストラクタ (ID なし)
WXBitmapPanel::WXBitmapPanel(wxWindow *parent, const wxSize& size, long style)
	: WXBitmapPanel(parent, wxID_ANY, size, style)	// 移譲
{
}

// コンストラクタ (ID あり)
WXBitmapPanel::WXBitmapPanel(wxWindow *parent, wxWindowID id,
	const wxSize& size, long style)
	: inherited(parent, id, wxDefaultPosition, size, style)
{
	bitmap_bgcolor = BGPANEL;
}

// デストラクタ
WXBitmapPanel::~WXBitmapPanel()
{
}

// 背景色
void
WXBitmapPanel::SetBitmapBGColor(Color c)
{
	bitmap_bgcolor = c;
	if (bitmap.GetBuf()) {
		bitmap.Fill(bitmap_bgcolor);
	}
}

// サイズ変更イベント
void
WXBitmapPanel::OnSize(wxSizeEvent& event)
{
	const wxSize& size = event.GetSize();
	if (size.x <= 1 || size.y <= 1) {
		return;
	}

	event.Skip();

	// バックバッファの大きさも変更
	wxSize oldsize(bitmap.GetWidth(), bitmap.GetHeight());
	if (size != oldsize) {
		bitmap.Create(size.x, size.y);
		bitmap.Fill(bitmap_bgcolor);
	}
}

// 描画イベント (UI スレッドから呼ばれる)
void
WXBitmapPanel::OnPaint(wxPaintEvent& event)
{
	if (__predict_false(bitmap.GetBuf() == NULL)) {
		return;
	}

	// 継承クラスによる描画
	Draw();

	wxImage image(bitmap.GetWidth(), bitmap.GetHeight(), bitmap.GetBuf(), true);
	wxBitmap bmp(image);

	// 実画面 DC にコピー
	wxPaintDC dstDC(this);
	dstDC.DrawBitmap(bmp, 0, 0);
}

// 描画処理
void
WXBitmapPanel::Draw()
{
	// 通常は継承クラスが記述する。
	// ここで用意している Draw() は何もしないことで、全域を背景色で
	// 塗りつぶしただけの状態が表示できる。空き地や下地表示用。
}
